@@ -53,12 +53,18 @@ pub struct LLVMCovSections {
 
 #[no_coverage]
 pub fn get_llvm_cov_sections(path: &Path) -> Result<LLVMCovSections, ReadCovMapError> {
-    let bin_data = std::fs::read(path).map_err(|_| ReadCovMapError::CannotReadObjectFile {
-        path: path.to_path_buf(),
-    })?;
-    let obj_file = object::File::parse(&*bin_data).map_err(|_| ReadCovMapError::CannotReadObjectFile {
-        path: path.to_path_buf(),
-    })?;
+    let bin_data = std::fs::read(path).map_err(
+        #[no_coverage]
+        |_| ReadCovMapError::CannotReadObjectFile {
+            path: path.to_path_buf(),
+        },
+    )?;
+    let obj_file = object::File::parse(&*bin_data).map_err(
+        #[no_coverage]
+        |_| ReadCovMapError::CannotReadObjectFile {
+            path: path.to_path_buf(),
+        },
+    )?;
     let covmap = obj_file
         .section_by_name("__llvm_covmap")
         .ok_or(ReadCovMapError::CannotFindSection {
@@ -241,10 +247,9 @@ fn read_mapping_regions(
     assert!(!covfun.is_empty());
 
     let mut result = Vec::new();
-
     for &filename_index in filename_indices {
         let num_regions = read_leb_usize(covfun, idx);
-        let mut prev_line_end = 0;
+        let mut prev_line_start = 0;
         for _ in 0..num_regions {
             let raw_header = read_leb_usize(covfun, idx);
             let header = read_counter(raw_header); //read_leb_usize(covfun, idx)); // counter or pseudo-counter
@@ -257,9 +262,9 @@ fn read_mapping_regions(
             let num_lines = read_leb_usize(covfun, idx);
             let col_end = read_leb_usize(covfun, idx);
 
-            let line_start = prev_line_end + delta_line_start;
+            let line_start = prev_line_start + delta_line_start;
             let line_end = line_start + num_lines;
-            prev_line_end = line_end;
+            prev_line_start = line_start;
             let file_region = MappingRegion {
                 filename_index,
                 line_start,
@@ -377,11 +382,17 @@ pub fn read_prf_data(prf_data: &[u8], idx: &mut usize) -> Result<Vec<PrfData>, R
 #[must_use]
 #[no_coverage]
 fn read_func_names(slice: &[u8], names: &mut Vec<String>) -> Result<(), ReadCovMapError> {
-    let slices = slice.split(|&x| x == 0x01);
+    let slices = slice.split(
+        #[no_coverage]
+        |&x| x == 0x01,
+    );
     for slice in slices {
-        let string = String::from_utf8(slice.to_vec()).map_err(|_| ReadCovMapError::CannotParseUTF8 {
-            section: CovMapSection::PrfNames,
-        })?;
+        let string = String::from_utf8(slice.to_vec()).map_err(
+            #[no_coverage]
+            |_| ReadCovMapError::CannotParseUTF8 {
+                section: CovMapSection::PrfNames,
+            },
+        )?;
         names.push(string);
     }
     Ok(())
@@ -435,12 +446,14 @@ pub fn process_function_records(
 ) -> Vec<FunctionRecord> {
     let mut all_expressions = Vec::new();
     for function_counters in records {
-        let mut expressions = HashMap::<ExpandedExpression, MappingRegion>::new();
+        let mut expressions = vec![];
+        // let mut expressions = HashMap::<ExpandedExpression, MappingRegion>::new();
         for (raw_counter, mapping_region) in function_counters.counters_list.iter() {
             let mut expanded = ExpandedExpression::default();
             expanded.push_counter(raw_counter, Sign::Positive, &function_counters);
             expanded.sort(); // sort them so that their hash is the same if they are equal
-            expressions.insert(expanded, mapping_region.clone());
+                             // expressions.insert(expanded, mapping_region.clone());
+            expressions.push((expanded, mapping_region.clone()));
         }
         let name_function = (&prf_names[&function_counters.header.id.name_md5]).clone();
 
@@ -469,7 +482,7 @@ pub fn process_function_records(
 pub enum CovMapSection {
     CovFun,
     CovMap,
-    PrfData,
+    // PrfData,
     PrfNames,
 }
 
@@ -480,7 +493,7 @@ pub enum ReadCovMapError {
         length: usize,
         cur_pointer: usize,
     },
-    FoundProfileDataForDummyFunction,
+    // FoundProfileDataForDummyFunction,
     InconsistentLengthOfEncodedData {
         section: CovMapSection,
     },
@@ -544,14 +557,17 @@ pub fn read_list_filenames(slice: &[u8], idx: &mut usize) -> Result<Vec<String>,
     let length_uncompressed = read_leb_usize(slice, idx);
     let length_compressed = read_leb_usize(slice, idx);
 
+    #[no_coverage]
     fn read_filenames(slice: &[u8], idx: &mut usize) -> Result<Vec<String>, ReadCovMapError> {
         let mut filenames = Vec::new();
         while *idx < slice.len() {
             let len = read_leb_usize(slice, idx);
-            let filename =
-                String::from_utf8(slice[*idx..*idx + len].to_vec()).map_err(|_| ReadCovMapError::CannotParseUTF8 {
+            let filename = String::from_utf8(slice[*idx..*idx + len].to_vec()).map_err(
+                #[no_coverage]
+                |_| ReadCovMapError::CannotParseUTF8 {
                     section: CovMapSection::CovMap,
-                })?;
+                },
+            )?;
             filenames.push(filename);
             *idx += len;
         }
@@ -649,7 +665,10 @@ impl ExpandedExpression {
             Sign::Negative => (&mut self.sub_terms, &mut self.add_terms),
             Sign::Positive => (&mut self.add_terms, &mut self.sub_terms),
         };
-        if let Some(index_in_counterpart) = counterpart.iter().position(|&x| x == term) {
+        if let Some(index_in_counterpart) = counterpart.iter().position(
+            #[no_coverage]
+            |&x| x == term,
+        ) {
             counterpart.remove(index_in_counterpart);
         } else {
             recipient.push(term);
@@ -676,19 +695,24 @@ impl ExpandedExpression {
     }
 }
 
-impl FunctionRecord {
-    #[no_coverage]
-    pub fn filter_trivial_expressions(&mut self) -> Vec<(usize, MappingRegion)> {
-        // filtered represent all the (unique) expressions that consist of only one positive term
-        // that term is necessarily a reference to one of the “physical” counters in __llvm_prf_cnts
-        let filtered = self
-            .expressions
-            .drain_filter(|(e, _)| e.add_terms.len() <= 1 && e.sub_terms.is_empty())
-            .filter_map(|(e, mapping_region)| e.add_terms.get(0).map(|c| (*c, mapping_region)))
-            .collect::<Vec<_>>();
-        filtered
-    }
-}
+// impl FunctionRecord {
+//     #[no_coverage]
+//     pub fn filter_trivial_expressions(&mut self) -> Vec<(usize, MappingRegion)> {
+//         // filtered represent all the (unique) expressions that consist of only one positive term
+//         // that term is necessarily a reference to one of the “physical” counters in __llvm_prf_cnts
+//         let filtered = self
+//             .expressions
+//             .drain_filter(|(e, _)| e.add_terms.len() <= 1 && e.sub_terms.is_empty())
+//             .filter_map(|(e, mapping_region)| {
+//                 e.add_terms.get(0).map(
+//                     #[no_coverage]
+//                     |c| (*c, mapping_region),
+//                 )
+//             })
+//             .collect::<Vec<_>>();
+//         filtered
+//     }
+// }
 
 pub struct OptimisedExpandedExpression {
     add_terms: Vec<*const u64>,
@@ -712,13 +736,13 @@ impl OptimisedExpandedExpression {
 
 impl ExpandedExpression {
     #[no_coverage]
-    fn optimized(&self, counters: &[u64]) -> OptimisedExpandedExpression {
-        assert!(
-            self.add_terms.len() > 1 || !self.sub_terms.is_empty(),
-            "{} {}",
-            self.add_terms.len(),
-            self.sub_terms.len()
-        );
+    fn optimised(&self, counters: &[u64]) -> OptimisedExpandedExpression {
+        // assert!(
+        //     self.add_terms.len() > 1 || !self.sub_terms.is_empty(),
+        //     "{} {}",
+        //     self.add_terms.len(),
+        //     self.sub_terms.len()
+        // );
         let mut add_terms = Vec::new();
         let mut sub_terms = Vec::new();
         for &add_term in &self.add_terms {
@@ -761,62 +785,61 @@ impl Coverage {
         let mut start_idx = 0;
         prf_datas
             .iter()
-            .map(|prf_data| {
-                let range = start_idx..start_idx + prf_data.number_of_counters;
-                start_idx = range.end;
-                let f_r = function_records
-                    .iter()
-                    .find(|fr| fr.header.id == prf_data.function_id)
-                    .ok_or(ReadCovMapError::CannotFindFunctionRecordAssociatedWithPrfData)?;
+            .map(
+                #[no_coverage]
+                |prf_data| {
+                    let range = start_idx..start_idx + prf_data.number_of_counters;
+                    start_idx = range.end;
+                    let f_r = function_records
+                        .iter()
+                        .find(
+                            #[no_coverage]
+                            |fr| fr.header.id == prf_data.function_id,
+                        )
+                        .ok_or(ReadCovMapError::CannotFindFunctionRecordAssociatedWithPrfData)?;
 
-                let slice = &mut all_counters[range];
-                let mut single_counters = Vec::new();
-                let mut expression_counters = Vec::new();
+                    let slice = &mut all_counters[range];
+                    let mut single_counters = Vec::new();
+                    let mut expression_counters = Vec::new();
 
-                for (e, _) in f_r.expressions.iter() {
-                    if e.add_terms.is_empty() && e.sub_terms.is_empty() {
-                        continue;
-                    } else if e.add_terms.len() == 1 && e.sub_terms.is_empty() {
-                        single_counters.push(&mut slice[e.add_terms[0]] as *mut _);
-                    } else if e.add_terms.len() > 0 {
-                        expression_counters.push(e.optimized(slice));
-                    } else {
-                        panic!(
-                            "An expression contains only sub terms\nAdd terms: {:?}\nSub terms: {:?}",
-                            e.add_terms, e.sub_terms
-                        );
+                    for (e, _) in f_r.expressions.iter() {
+                        if e.add_terms.is_empty() && e.sub_terms.is_empty() {
+                            continue;
+                        } else if e.add_terms.len() == 1 && e.sub_terms.is_empty() {
+                            single_counters.push(&mut slice[e.add_terms[0]] as *mut _);
+                        } else if e.add_terms.len() > 0 {
+                            expression_counters.push(e.optimised(slice));
+                        } else {
+                            panic!(
+                                "An expression contains only sub terms\nAdd terms: {:?}\nSub terms: {:?}",
+                                e.add_terms, e.sub_terms
+                            );
+                        }
                     }
-                }
-                // let nbr_expressions = expression_counters.len();
-                single_counters.sort();
-                Ok(Coverage {
-                    function_record: f_r.clone(),
-                    start_counters: slice.as_mut_ptr(),
-                    counters_len: slice.len(),
-                    single_counters,
-                    expression_counters,
-                    // computed_expressions: Vec::with_capacity(nbr_expressions),
-                })
-            })
+                    Ok(Coverage {
+                        function_record: f_r.clone(),
+                        start_counters: slice.as_mut_ptr(),
+                        counters_len: slice.len(),
+                        single_counters,
+                        expression_counters,
+                    })
+                },
+            )
             .collect()
     }
     #[no_coverage]
-    pub(crate) fn filter_function_by_files<F, G>(all_self: &mut Vec<Self>, exclude_f: F, keep_f: G)
+    pub(crate) fn filter_function_by_files<F>(all_self: &mut Vec<Self>, keep_f: F)
     where
         F: Fn(&Path) -> bool,
-        G: Fn(&Path) -> bool,
     {
-        all_self.drain_filter(|coverage| {
-            let mut excluded = false;
-            for filepath in &coverage.function_record.filenames {
-                if keep_f(filepath) {
-                    return false;
+        all_self.drain_filter(
+            #[no_coverage]
+            |coverage| {
+                for filepath in &coverage.function_record.filenames {
+                    return !keep_f(filepath);
                 }
-                if exclude_f(filepath) {
-                    excluded = true;
-                }
-            }
-            excluded
-        });
+                return false;
+            },
+        );
     }
 }
