@@ -9,7 +9,6 @@ use std::fmt::Debug;
 use std::hash::Hash;
 use std::ops::Index;
 use std::ops::IndexMut;
-use std::ops::Range;
 
 // ========= Slab ============
 
@@ -144,15 +143,15 @@ impl<T> Slab<T> {
     //         SlabKey::new(self.storage.len())
     //     }
     // }
-    #[no_coverage]
-    pub fn get_mut(&mut self, key: SlabKey<T>) -> Option<&mut T> {
-        // O(n) but in practice very fast because there will be almost no available slots
-        if self.available_slots.contains(&key.key) {
-            None
-        } else {
-            Some(&mut self.storage[key.key])
-        }
-    }
+    // #[no_coverage]
+    // pub fn get_mut(&mut self, key: SlabKey<T>) -> Option<&mut T> {
+    //     // O(n) but in practice very fast because there will be almost no available slots
+    //     if self.available_slots.contains(&key.key) {
+    //         None
+    //     } else {
+    //         Some(&mut self.storage[key.key])
+    //     }
+    // }
 }
 
 impl<T> Index<SlabKey<T>> for Slab<T> {
@@ -173,53 +172,136 @@ impl<T> IndexMut<SlabKey<T>> for Slab<T> {
 
 impl<T> Slab<T> {
     #[no_coverage]
-    pub fn keys(&self) -> impl Iterator<Item = SlabKey<T>> + '_ {
+    pub fn keys(&self) -> impl Iterator<Item = SlabKey<T>> {
+        let available_slots = self.available_slots.clone();
         (0..self.storage.len())
             .into_iter()
-            .filter(move |i| !self.available_slots.contains(i))
-            .map(|raw_key| SlabKey::new(raw_key))
+            .filter(
+                #[no_coverage]
+                move |i| !available_slots.contains(i),
+            )
+            .map(
+                #[no_coverage]
+                |raw_key| SlabKey::new(raw_key),
+            )
     }
 }
 
-// ========== WeightedIndex ===========
-/// Generate a random f64 within the given range
-/// The start and end of the range must be finite
-/// This is a very naive implementation
-#[inline(always)]
-#[no_coverage]
-fn gen_f64(rng: &fastrand::Rng, range: Range<f64>) -> f64 {
-    range.start + rng.f64() * (range.end - range.start)
+// ========== RcSlab ===========
+// Like a Slab, but each entry is ref-counted
+
+#[derive(Debug)]
+struct RcSlabSlot<T> {
+    data: T,
+    ref_count: usize,
 }
 
 /**
- * A distribution using weighted sampling to pick a discretely selected item.
- *
- * An alternative implementation of the same type by the `rand` crate.
+ * Pre-allocated reference-counted storage for a uniform data type.
  */
-#[derive(Debug, Clone)]
-pub struct WeightedIndex<'a> {
-    pub cumulative_weights: &'a Vec<f64>,
+pub struct RcSlab<T> {
+    storage: Vec<RcSlabSlot<T>>,
+    available_slots: Vec<usize>,
 }
 
-impl<'a> WeightedIndex<'a> {
+impl<T: Debug> Debug for RcSlab<T> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        let storage = self.keys().map(|k| &self[k]).collect::<Vec<_>>();
+        f.debug_struct("Slab")
+            .field("storage", &storage)
+            .field("available_slots", &self.available_slots.len())
+            .finish()
+    }
+}
+
+impl<T> RcSlab<T> {
+    #[no_coverage]
+    pub fn new() -> Self {
+        Self {
+            storage: Vec::with_capacity(1000),
+            available_slots: Vec::with_capacity(32),
+        }
+    }
+    // #[no_coverage]
+    // pub fn len(&self) -> usize {
+    //     self.storage.len() - self.available_slots.len()
+    // }
+    // #[no_coverage]
+    // pub fn is_empty(&self) -> bool {
+    //     self.len() == 0
+    // }
+    // #[no_coverage]
+    // pub fn get_nth_key(&self, n: usize) -> SlabKey<T> {
+    //     let mut idx = n;
+    //     for &i in &self.available_slots {
+    //         if i <= idx {
+    //             idx += 1;
+    //         }
+    //     }
+    //     SlabKey::new(idx)
+    // }
+
+    #[no_coverage]
+    pub fn insert(&mut self, x: T, ref_count: usize) -> usize {
+        if let Some(&slot) = self.available_slots.last() {
+            self.available_slots.pop();
+            self.storage[slot] = RcSlabSlot { data: x, ref_count };
+            slot
+        } else {
+            self.storage.push(RcSlabSlot { data: x, ref_count });
+            self.storage.len() - 1
+        }
+    }
+    #[no_coverage]
+    pub fn remove(&mut self, key: usize) {
+        let slot = &mut self.storage[key];
+        assert!(slot.ref_count > 0);
+        slot.ref_count -= 1;
+        if slot.ref_count == 0 {
+            self.available_slots.push(key);
+            self.available_slots.sort();
+        }
+    }
+    #[no_coverage]
+    pub fn next_slot(&self) -> usize {
+        if let Some(&slot) = self.available_slots.last() {
+            slot
+        } else {
+            self.storage.len()
+        }
+    }
     #[no_coverage]
-    pub fn sample(&self, rng: &fastrand::Rng) -> usize {
-        assert!(!self.cumulative_weights.is_empty());
-        if self.cumulative_weights.len() == 1 {
-            return 0;
+    pub fn get_mut(&mut self, key: usize) -> Option<&mut T> {
+        // O(n) but in practice very fast because there will be almost no available slots
+        if self.available_slots.contains(&key) {
+            None
+        } else {
+            Some(&mut self.storage[key].data)
         }
+    }
+}
 
-        let range = 0.0..*self.cumulative_weights.last().unwrap();
-        let chosen_weight = gen_f64(rng, range);
-        // Find the first item which has a weight *higher* than the chosen weight.
-        self.cumulative_weights
-            .binary_search_by(|w| {
-                if *w <= chosen_weight {
-                    Ordering::Less
-                } else {
-                    Ordering::Greater
-                }
-            })
-            .unwrap_err()
+impl<T> Index<usize> for RcSlab<T> {
+    type Output = T;
+    #[inline(always)]
+    #[no_coverage]
+    fn index(&self, key: usize) -> &Self::Output {
+        &self.storage[key].data
+    }
+}
+impl<T> IndexMut<usize> for RcSlab<T> {
+    #[inline(always)]
+    #[no_coverage]
+    fn index_mut(&mut self, key: usize) -> &mut Self::Output {
+        &mut self.storage[key].data
+    }
+}
+
+impl<T> RcSlab<T> {
+    #[no_coverage]
+    pub fn keys(&self) -> impl Iterator<Item = usize> + '_ {
+        (0..self.storage.len())
+            .into_iter()
+            .filter(move |i| !self.available_slots.contains(i))
     }
 }
