@@ -3,87 +3,110 @@
 //! test inputs.
 
 use crate::code_coverage_sensor::CodeCoverageSensor;
-use crate::mutators::either::Either;
-use crate::sensors_and_pools::and_sensor_and_pool::{AndPool, AndSensor};
-use crate::sensors_and_pools::artifacts_pool::{ArtifactsPool, TestFailure, TestFailureSensor, TEST_FAILURE};
-use crate::sensors_and_pools::noop_sensor::NoopSensor;
-use crate::sensors_and_pools::unique_coverage_pool::UniqueCoveragePool;
-use crate::sensors_and_pools::unit_pool::UnitPool;
+use crate::data_structures::RcSlab;
+use crate::sensors_and_pools::stats::EmptyStats;
+use crate::sensors_and_pools::NoopSensor;
+use crate::sensors_and_pools::SimplestToActivateCounterPool;
+use crate::sensors_and_pools::UnitPool;
+use crate::sensors_and_pools::{AndPool, AndSensor};
+use crate::sensors_and_pools::{TestFailure, TestFailurePool, TestFailureSensor, TEST_FAILURE};
 use crate::signals_handler::set_signal_handlers;
-use crate::traits::{CompatibleWithSensor, EmptyStats, Pool, Sensor};
+use crate::traits::SaveToStatsFolder;
+use crate::traits::{CompatibleWithSensor, CorpusDelta, Pool, Sensor};
 use crate::traits::{Mutator, Serializer};
 use crate::world::World;
-use crate::FuzzedInput;
+use crate::{CSVField, FuzzedInput, ToCSV};
 use fuzzcheck_common::arg::{Arguments, FuzzerCommand};
 use fuzzcheck_common::{FuzzerEvent, FuzzerStats};
 use libc::{SIGABRT, SIGALRM, SIGBUS, SIGFPE, SIGINT, SIGSEGV, SIGTERM, SIGTRAP};
 use std::borrow::Borrow;
 use std::collections::hash_map::DefaultHasher;
 use std::hash::{Hash, Hasher};
-use std::panic::{catch_unwind, RefUnwindSafe, UnwindSafe};
+use std::panic::{catch_unwind, AssertUnwindSafe};
 use std::process::exit;
 use std::result::Result;
 
-#[derive(Debug)]
-pub enum ReasonForStopping<T> {
-    MaximumIterations,
-    MaximumDuration,
-    ExhaustedPossibleMutations,
-    TestFailure(T),
-    IOError(std::io::Error),
+static WRITE_STATS_ERROR: &str = "the stats could not be written to the file system";
+static WORLD_NEW_ERROR: &str = "an IO operation failed when setting up the fuzzer";
+static SERIALIZER_FROM_DATA_ERROR: &str = "the file could not be decoded into a valid input";
+static READ_INPUT_FILE_ERROR: &str = "the input file could not be read";
+static SAVE_ARTIFACTS_ERROR: &str = "the artifact could not be saved";
+static UPDATE_CORPUS_ERROR: &str = "the corpus could not be updated on the file system";
+
+static mut DID_FIND_ANY_TEST_FAILURE: bool = false;
+
+#[derive(Debug, Clone)]
+pub struct FuzzingResult<T> {
+    pub found_test_failure: bool,
+    pub reason_for_stopping: ReasonForStopping<T>,
 }
 
-impl<T> From<std::io::Error> for ReasonForStopping<T> {
-    fn from(e: std::io::Error) -> Self {
-        Self::IOError(e)
+#[derive(Debug, Clone)]
+pub enum ReasonForStopping<T> {
+    TestFailure(T),
+    ExhaustedAllPossibleMutations,
+    MaxIterationsReached,
+    MaxDurationReached,
+    LaunchedFuzzcheckWithoutCfgFuzzing,
+}
+
+/// The index to a test case in the fuzzer’s storage.
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash, serde::Serialize, serde::Deserialize)]
+pub struct PoolStorageIndex(usize);
+
+#[cfg(test)]
+impl PoolStorageIndex {
+    pub fn mock(idx: usize) -> Self {
+        Self(idx)
     }
 }
 
-enum FuzzerInputIndex<T, PoolIndex> {
+enum FuzzerInputIndex<T> {
     None,
     Temporary(T),
-    Pool(PoolIndex),
+    Pool(PoolStorageIndex),
 }
 
-struct FuzzerState<T: Clone, M: Mutator<T>, S: Serializer<Value = T>, Sens: Sensor, P: Pool>
+struct FuzzerState<T: Clone, M: Mutator<T>, Sens: Sensor, P: Pool>
 where
     P: CompatibleWithSensor<Sens>,
 {
     mutator: M,
     sensor: Sens,
+    pool_storage: RcSlab<FuzzedInput<T, M>>,
     pool: P,
     /// The step given to the mutator when the fuzzer wants to create a new arbitrary test case
     arbitrary_step: M::ArbitraryStep,
     /// The index of the test case that is being tested
-    input_idx: FuzzerInputIndex<FuzzedInput<T, M>, P::Index>,
+    input_idx: FuzzerInputIndex<FuzzedInput<T, M>>,
     /// Various statistics about the fuzzer run
     fuzzer_stats: FuzzerStats,
 
     settings: Arguments,
+    serializer: Box<dyn Serializer<Value = T>>,
     /// The world handles effects
-    world: World<S, P::Index>,
+    world: World,
 }
 
-impl<T: Clone, M: Mutator<T>, S: Serializer<Value = T>, Sens: Sensor, P: Pool<TestCase = FuzzedInput<T, M>>>
-    FuzzerState<T, M, S, Sens, P>
+impl<T: Clone, M: Mutator<T>, Sens: Sensor, P: Pool> FuzzerState<T, M, Sens, P>
 where
     P: CompatibleWithSensor<Sens>,
 {
     #[no_coverage]
     fn get_input<'a>(
-        fuzzer_input_idx: &'a FuzzerInputIndex<FuzzedInput<T, M>, P::Index>,
-        pool: &'a P,
+        fuzzer_input_idx: &'a FuzzerInputIndex<FuzzedInput<T, M>>,
+        pool_storage: &'a RcSlab<FuzzedInput<T, M>>,
     ) -> Option<&'a FuzzedInput<T, M>> {
         match fuzzer_input_idx {
             FuzzerInputIndex::None => None,
             FuzzerInputIndex::Temporary(input) => Some(input),
-            FuzzerInputIndex::Pool(idx) => Some(pool.get(*idx)),
+            FuzzerInputIndex::Pool(idx) => Some(&pool_storage[idx.0]),
         }
     }
 }
 
 #[no_coverage]
-fn update_fuzzer_stats<B: Serializer, C: Hash + Eq>(stats: &mut FuzzerStats, world: &mut World<B, C>) {
+fn update_fuzzer_stats(stats: &mut FuzzerStats, world: &mut World) {
     let microseconds = world.elapsed_time_since_last_checkpoint();
     let nbr_runs = stats.total_number_of_runs - stats.number_of_runs_since_last_reset_time;
     let nbr_runs_times_million = nbr_runs * 1_000_000;
@@ -95,12 +118,30 @@ fn update_fuzzer_stats<B: Serializer, C: Hash + Eq>(stats: &mut FuzzerStats, wor
     }
 }
 
-impl<T: Clone, M: Mutator<T>, S: Serializer<Value = T>, Sens: Sensor, P: Pool<TestCase = FuzzedInput<T, M>>>
-    FuzzerState<T, M, S, Sens, P>
+impl<T: Clone, M: Mutator<T>, Sens: Sensor, P: Pool> SaveToStatsFolder for FuzzerState<T, M, Sens, P>
 where
     P: CompatibleWithSensor<Sens>,
     Self: 'static,
 {
+    #[no_coverage]
+    fn save_to_stats_folder(&self) -> Vec<(std::path::PathBuf, Vec<u8>)> {
+        let mut contents = self.pool.save_to_stats_folder();
+        contents.extend(self.sensor.save_to_stats_folder());
+        contents.extend(self.world.save_to_stats_folder());
+        contents
+    }
+}
+
+impl<T: Clone, M: Mutator<T>, Sens: Sensor, P: Pool> FuzzerState<T, M, Sens, P>
+where
+    P: CompatibleWithSensor<Sens>,
+    Self: 'static,
+{
+    #[no_coverage]
+    fn write_stats(&mut self) -> Result<(), std::io::Error> {
+        self.world.write_stats_content(self.save_to_stats_folder())
+    }
+
     #[no_coverage]
     fn receive_signal(&mut self, signal: i32) -> ! {
         self.world.report_event(
@@ -110,10 +151,11 @@ where
 
         match signal {
             SIGABRT | SIGBUS | SIGSEGV | SIGFPE | SIGALRM | SIGTRAP => {
-                if let Some(input) = Self::get_input(&self.input_idx, &self.pool) {
+                if let Some(input) = Self::get_input(&self.input_idx, &self.pool_storage) {
                     let cplx = input.complexity(&self.mutator);
-                    let _ = self.world.save_artifact(&input.value, cplx);
-
+                    let content = self.serializer.to_data(&input.value);
+                    let _ = self.world.save_artifact(content, cplx, self.serializer.extension());
+                    self.write_stats().expect(WRITE_STATS_ERROR);
                     exit(TerminationStatus::Crash as i32);
                 } else {
                     self.world.report_event(
@@ -123,7 +165,10 @@ where
                     exit(TerminationStatus::Crash as i32);
                 }
             }
-            SIGINT | SIGTERM => self.world.stop(),
+            SIGINT | SIGTERM => {
+                self.write_stats().expect(WRITE_STATS_ERROR);
+                self.world.stop()
+            }
             _ => exit(TerminationStatus::Unknown as i32),
         }
     }
@@ -142,56 +187,60 @@ where
     #[no_coverage]
     unsafe fn set_up_signal_handler(&mut self) {
         let ptr = self as *mut Self;
-        set_signal_handlers(move |sig| (&mut *ptr).receive_signal(sig));
+        set_signal_handlers(
+            #[no_coverage]
+            move |sig| (&mut *ptr).receive_signal(sig),
+        );
     }
 }
 
-pub struct Fuzzer<T, FT, F, M, S, Sens, P>
+pub struct Fuzzer<T, M, Sens, P>
 where
-    FT: ?Sized,
-    T: Clone + Borrow<FT>,
-    F: Fn(&FT) -> bool,
+    T: Clone,
     M: Mutator<T>,
-    S: Serializer<Value = T>,
     Sens: Sensor,
     P: Pool,
     P: CompatibleWithSensor<Sens>,
     Self: 'static,
 {
-    state: FuzzerState<T, M, S, Sens, P>,
-    test: F,
-    phantom: std::marker::PhantomData<FT>,
+    state: FuzzerState<T, M, Sens, P>,
+    test: Box<dyn Fn(&T) -> bool>,
 }
 
-impl<T, FT, F, M, S, Sens, P> Fuzzer<T, FT, F, M, S, Sens, P>
+impl<T, M, Sens, P> Fuzzer<T, M, Sens, P>
 where
-    FT: ?Sized,
-    T: Clone + Borrow<FT>,
-    F: Fn(&FT) -> bool,
+    T: Clone,
     M: Mutator<T>,
-    S: Serializer<Value = T>,
     Sens: Sensor,
-    P: Pool<TestCase = FuzzedInput<T, M>>,
+    P: Pool,
     P: CompatibleWithSensor<Sens>,
     Self: 'static,
-    P::Stats: Default,
 {
     #[no_coverage]
-    fn new(test: F, mutator: M, sensor: Sens, pool: P, settings: Arguments, world: World<S, P::Index>) -> Self {
+    fn new(
+        test: Box<dyn Fn(&T) -> bool>,
+        mutator: M,
+        serializer: Box<dyn Serializer<Value = T>>,
+        sensor: Sens,
+        pool: P,
+        settings: Arguments,
+        world: World,
+    ) -> Self {
         let arbitrary_step = mutator.default_arbitrary_step();
         Fuzzer {
             state: FuzzerState {
                 sensor,
+                pool_storage: RcSlab::new(),
                 pool,
                 mutator,
                 arbitrary_step,
                 input_idx: FuzzerInputIndex::None,
                 fuzzer_stats: FuzzerStats::default(),
                 settings,
+                serializer,
                 world,
             },
             test,
-            phantom: std::marker::PhantomData,
         }
     }
 
@@ -202,9 +251,11 @@ where
                 FuzzerState {
                     mutator,
                     sensor,
+                    pool_storage,
                     pool,
                     input_idx,
                     fuzzer_stats,
+                    serializer,
                     world,
                     ..
                 },
@@ -213,25 +264,27 @@ where
         } = self;
 
         // we have verified in the caller function that there is an input
-        let input = FuzzerState::<T, M, S, Sens, P>::get_input(input_idx, pool).unwrap();
+        let input = FuzzerState::<T, M, Sens, P>::get_input(input_idx, pool_storage).unwrap();
 
-        std::panic::set_hook(Box::new(move |panic_info| {
-            let mut hasher = DefaultHasher::new();
-            panic_info.location().hash(&mut hasher);
-            unsafe {
-                TEST_FAILURE = Some(TestFailure {
-                    display: format!("{}", panic_info),
-                    id: hasher.finish(),
-                });
-            }
-        }));
+        std::panic::set_hook(Box::new(
+            #[no_coverage]
+            move |panic_info| {
+                let mut hasher = DefaultHasher::new();
+                panic_info.location().hash(&mut hasher);
+                unsafe {
+                    TEST_FAILURE = Some(TestFailure {
+                        display: format!("{}", panic_info),
+                        id: hasher.finish(),
+                    });
+                }
+            },
+        ));
 
         sensor.start_recording();
-        let cell = NotUnwindSafe { value: test };
-        let input_cell = NotUnwindSafe {
-            value: input.value.borrow(),
-        };
-        let result = catch_unwind(|| (cell.value)(input_cell.value));
+        let result = catch_unwind(AssertUnwindSafe(
+            #[no_coverage]
+            || (test)(input.value.borrow()),
+        ));
         let _ = std::panic::take_hook();
         let test_failure = match result {
             Ok(false) => unsafe {
@@ -248,51 +301,87 @@ where
             }
             Ok(true) => false,
         };
+        if test_failure {
+            unsafe {
+                DID_FIND_ANY_TEST_FAILURE = true;
+            }
+        }
         sensor.stop_recording();
         if test_failure && self.state.settings.stop_after_first_failure {
-            self.state.world.save_artifact(&input.value, cplx)?;
+            let serialized_input = serializer.to_data(&input.value);
+            self.state
+                .world
+                .save_artifact(serialized_input, cplx, serializer.extension())
+                .expect(SAVE_ARTIFACTS_ERROR);
             return Err(ReasonForStopping::TestFailure(input.value.clone()));
         }
 
         fuzzer_stats.total_number_of_runs += 1;
 
-        let get_input = match &input_idx {
-            FuzzerInputIndex::None => unreachable!(),
-            FuzzerInputIndex::Temporary(input) => Either::Right(input),
-            FuzzerInputIndex::Pool(idx) => Either::Left(*idx),
-        };
-        let clone_input = |input: &FuzzedInput<T, M>| input.new_source(mutator);
+        let input_id = PoolStorageIndex(pool_storage.next_slot());
 
-        pool.process(sensor, get_input, &clone_input, cplx, |corpus_delta, pool_stats| {
-            let corpus_delta = corpus_delta.convert(|x| x.value.clone());
+        let deltas = pool.process(input_id, sensor, cplx);
+
+        if !deltas.is_empty() {
+            let add_ref_count = deltas.iter().fold(
+                0,
+                #[no_coverage]
+                |acc, delta| if delta.add { acc + 1 } else { acc },
+            );
             update_fuzzer_stats(fuzzer_stats, world);
-            let event = corpus_delta.fuzzer_event();
-            world.update_corpus(corpus_delta)?;
-            world.report_event(event, Some((fuzzer_stats, pool_stats)));
-            Ok(())
-        })?;
+            let event = CorpusDelta::fuzzer_event(&deltas);
+            let content = if add_ref_count > 0 {
+                serializer.to_data(&input.value)
+            } else {
+                vec![]
+            };
+            world
+                .update_corpus(input_id, content, &deltas, serializer.extension())
+                .expect(UPDATE_CORPUS_ERROR);
+            world.report_event(event, Some((fuzzer_stats, &pool.stats())));
+            if add_ref_count > 0 {
+                let new_input = input.new_source(mutator);
+                // here I don't check the complexity of the new input,
+                // but because of the way mutators work (real possibility of
+                // inconsistent complexities), then its complexity may be higher
+                // than the maximum allowed one
+                pool_storage.insert(new_input, add_ref_count);
+            }
+            for delta in deltas {
+                for r in delta.remove {
+                    pool_storage.remove(r.0);
+                }
+            }
+        }
 
         Ok(())
     }
 
     #[no_coverage]
     fn process_next_input(&mut self) -> Result<(), ReasonForStopping<T>> {
-        let pool = &mut self.state.pool;
+        let FuzzerState {
+            pool_storage,
+            pool,
+            input_idx,
+            mutator,
+            settings,
+            ..
+        } = &mut self.state;
         loop {
             if let Some(idx) = pool.get_random_index() {
-                self.state.input_idx = FuzzerInputIndex::Pool(idx);
-                let input = pool.get_mut(idx);
+                *input_idx = FuzzerInputIndex::Pool(idx);
+                let input = &mut pool_storage[idx.0];
                 let generation = input.generation;
-                if let Some((unmutate_token, cplx)) =
-                    input.mutate(&mut self.state.mutator, self.state.settings.max_input_cplx)
-                {
+                if let Some((unmutate_token, cplx)) = input.mutate(mutator, settings.max_input_cplx) {
                     if cplx < self.state.settings.max_input_cplx {
                         self.test_and_process_input(cplx)?;
                     }
-                    let pool = &mut self.state.pool;
+
                     // Retrieving the input may fail because the input may have been deleted
-                    if let Some(input) = pool.retrieve_after_processing(idx, generation) {
-                        input.unmutate(&self.state.mutator, unmutate_token);
+                    if let Some(input) = self.state.pool_storage.get_mut(idx.0) {
+                        if input.generation == generation {
+                            input.unmutate(&self.state.mutator, unmutate_token);
+                        }
                     }
 
                     break Ok(());
@@ -313,7 +402,7 @@ where
                     FuzzerEvent::End,
                     Some((&self.state.fuzzer_stats, &self.state.pool.stats())),
                 );
-                break Err(ReasonForStopping::ExhaustedPossibleMutations);
+                break Err(ReasonForStopping::ExhaustedAllPossibleMutations);
             }
         }
     }
@@ -323,15 +412,17 @@ where
         let mut inputs: Vec<FuzzedInput<T, M>> = self
             .state
             .world
-            .read_input_corpus()?
+            .read_input_corpus()
+            .expect(READ_INPUT_FILE_ERROR)
             .into_iter()
-            .filter_map(|value| {
-                if let Some((cache, mutation_step)) = self.state.mutator.validate_value(&value) {
+            .filter_map(
+                #[no_coverage]
+                |value| {
+                    let value = self.state.serializer.from_data(&value)?;
+                    let (cache, mutation_step) = self.state.mutator.validate_value(&value)?;
                     Some(FuzzedInput::new(value, cache, mutation_step, 0))
-                } else {
-                    None
-                }
-            })
+                },
+            )
             .collect();
 
         for _ in 0..100 {
@@ -342,7 +433,10 @@ where
             }
         }
 
-        inputs.drain_filter(|i| i.complexity(&self.state.mutator) > self.state.settings.max_input_cplx);
+        inputs.drain_filter(
+            #[no_coverage]
+            |i| i.complexity(&self.state.mutator) > self.state.settings.max_input_cplx,
+        );
         assert!(!inputs.is_empty());
 
         self.state.world.set_checkpoint_instant();
@@ -367,15 +461,15 @@ where
             Some((&self.state.fuzzer_stats, &self.state.pool.stats())),
         );
 
-        self.state.world.set_start_instant();
+        self.state.world.set_checkpoint_instant();
         let mut next_milestone = (self.state.fuzzer_stats.total_number_of_runs + 100_000) * 2;
         loop {
             let duration_since_beginning = self.state.world.elapsed_time_since_start();
             if duration_since_beginning > self.state.settings.maximum_duration {
-                return Err(ReasonForStopping::MaximumDuration);
+                return Err(ReasonForStopping::MaxDurationReached);
             }
             if self.state.fuzzer_stats.total_number_of_runs >= self.state.settings.maximum_iterations {
-                return Err(ReasonForStopping::MaximumIterations);
+                return Err(ReasonForStopping::MaxIterationsReached);
             }
             self.process_next_input()?;
             if self.state.fuzzer_stats.total_number_of_runs >= next_milestone {
@@ -388,48 +482,8 @@ where
             }
         }
     }
-
-    /// Reads a corpus of inputs from the [World] and minifies the corpus
-    /// such that only the highest-scoring inputs are kept.
-    ///
-    /// The number of inputs to keep is taken from
-    /// [`self.settings.corpus_size`](FuzzerSettings::corpus_size)
-    #[no_coverage]
-    fn corpus_minifying_loop(&mut self, corpus_size: usize) -> Result<(), ReasonForStopping<T>> {
-        self.state.world.report_event(
-            FuzzerEvent::Start,
-            Some((&self.state.fuzzer_stats, &self.state.pool.stats())),
-        );
-        self.process_initial_inputs()?;
-
-        let FuzzerState {
-            pool,
-            fuzzer_stats,
-            world,
-            ..
-        } = &mut self.state;
-
-        world.report_event(FuzzerEvent::DidReadCorpus, Some((fuzzer_stats, pool.stats().clone())));
-
-        pool.minify(corpus_size, |corpus_delta, pool_stats| {
-            let corpus_delta = corpus_delta.convert(|x| x.value.clone());
-            let event = corpus_delta.fuzzer_event();
-            world.update_corpus(corpus_delta)?;
-            world.report_event(event, Some((fuzzer_stats, pool_stats)));
-            Ok(())
-        })?;
-        world.report_event(FuzzerEvent::Done, Some((fuzzer_stats, pool.stats())));
-        Ok(())
-    }
 }
 
-struct NotUnwindSafe<T> {
-    value: T,
-}
-
-impl<T> UnwindSafe for NotUnwindSafe<T> {}
-impl<T> RefUnwindSafe for NotUnwindSafe<T> {}
-
 pub enum TerminationStatus {
     Success = 0,
     Crash = 1,
@@ -438,115 +492,146 @@ pub enum TerminationStatus {
 }
 
 #[no_coverage]
-pub fn launch<T, FT, F, M, S, Sens, P>(
-    test: F,
+pub fn launch<T, M, Sens, P>(
+    test: Box<dyn Fn(&T) -> bool>,
     mutator: M,
-    serializer: S,
+    serializer: Box<dyn Serializer<Value = T>>,
     sensor: Sens,
     pool: P,
     mut args: Arguments,
-) -> Result<(), ReasonForStopping<T>>
+) -> FuzzingResult<T>
 where
-    FT: ?Sized,
-    T: Clone + Borrow<FT>,
-    F: Fn(&FT) -> bool,
+    T: Clone,
     M: Mutator<T>,
-    S: Serializer<Value = T>,
-    Fuzzer<T, FT, F, M, S, CodeCoverageSensor, UniqueCoveragePool<FuzzedInput<T, M>>>: 'static,
+    Fuzzer<T, M, CodeCoverageSensor, SimplestToActivateCounterPool>: 'static,
     Sens: Sensor + 'static,
-    P: Pool<TestCase = FuzzedInput<T, M>> + CompatibleWithSensor<Sens> + 'static,
+    P: Pool + CompatibleWithSensor<Sens> + 'static,
 {
     let command = &args.command;
-    let result = match command {
+    let reason_for_stopping = match command {
         FuzzerCommand::Fuzz => {
-            let test_failure = TestFailureSensor::default();
-            let sensor = AndSensor {
-                s1: sensor,
-                s2: test_failure,
-            };
-            let artifacts_pool = ArtifactsPool::new("artifacts");
-            let pool = AndPool::new(pool, artifacts_pool, 254);
-            let mut fuzzer = Fuzzer::new(
-                test,
-                mutator,
-                sensor,
-                pool,
-                args.clone(),
-                World::new(serializer, args.clone()),
-            );
-            unsafe { fuzzer.state.set_up_signal_handler() };
-            fuzzer.main_loop()?
+            if !args.stop_after_first_failure {
+                let test_failure = TestFailureSensor::default();
+                let sensor = AndSensor(sensor, test_failure);
+                let test_failure_pool = TestFailurePool::new("test_failures");
+                let pool = AndPool::new(pool, test_failure_pool, 254);
+                let mut fuzzer = Fuzzer::new(
+                    test,
+                    mutator,
+                    serializer,
+                    sensor,
+                    pool,
+                    args.clone(),
+                    World::new(args.clone()).expect(WORLD_NEW_ERROR),
+                );
+
+                let mut stats_headers = vec![CSVField::String("time".to_string())];
+                stats_headers.extend(fuzzer.state.fuzzer_stats.csv_headers());
+                stats_headers.extend(fuzzer.state.pool.stats().csv_headers());
+                fuzzer
+                    .state
+                    .world
+                    .append_stats_file(&stats_headers)
+                    .expect(WRITE_STATS_ERROR);
+                unsafe { fuzzer.state.set_up_signal_handler() };
+
+                let reason_for_stopping = fuzzer.main_loop().unwrap_err();
+                fuzzer.state.write_stats().expect(WRITE_STATS_ERROR);
+
+                reason_for_stopping
+            } else {
+                let mut fuzzer = Fuzzer::new(
+                    test,
+                    mutator,
+                    serializer,
+                    sensor,
+                    pool,
+                    args.clone(),
+                    World::new(args.clone()).expect(WORLD_NEW_ERROR),
+                );
+                unsafe { fuzzer.state.set_up_signal_handler() };
+
+                let mut stats_headers = vec![CSVField::String("time".to_string())];
+                stats_headers.extend(fuzzer.state.fuzzer_stats.csv_headers());
+                stats_headers.extend(fuzzer.state.pool.stats().csv_headers());
+                fuzzer
+                    .state
+                    .world
+                    .append_stats_file(&stats_headers)
+                    .expect(WRITE_STATS_ERROR);
+                let reason_for_stopping = fuzzer.main_loop().unwrap_err();
+                fuzzer.state.write_stats().expect(WRITE_STATS_ERROR);
+
+                reason_for_stopping
+            }
         }
         FuzzerCommand::MinifyInput { input_file } => {
-            let world = World::new(serializer, args.clone());
-            let value = world.read_input_file(input_file)?;
+            let world = World::new(args.clone()).expect(WORLD_NEW_ERROR);
+            let value = world.read_input_file(input_file).expect(READ_INPUT_FILE_ERROR);
+            let value = serializer.from_data(&value).expect(SERIALIZER_FROM_DATA_ERROR);
             if let Some((cache, mutation_step)) = mutator.validate_value(&value) {
                 args.max_input_cplx = mutator.complexity(&value, &cache) - 0.01;
 
-                let sensor = AndSensor {
-                    s1: sensor,
-                    s2: NoopSensor,
-                };
-                let unit_pool = UnitPool::new(FuzzedInput::new(value, cache, mutation_step, 0));
+                let sensor = AndSensor(sensor, NoopSensor);
+
+                let unit_pool = UnitPool::new(PoolStorageIndex(0));
                 let pool = AndPool::new(pool, unit_pool, 240);
-                let mut fuzzer = Fuzzer::<_, _, _, _, _, _, _>::new(test, mutator, sensor, pool, args.clone(), world);
+                let mut fuzzer = Fuzzer::new(test, mutator, serializer, sensor, pool, args.clone(), world);
+                fuzzer
+                    .state
+                    .pool_storage
+                    .insert(FuzzedInput::new(value, cache, mutation_step, 0), 1);
 
                 unsafe { fuzzer.state.set_up_signal_handler() };
 
-                fuzzer.main_loop()?
+                fuzzer.main_loop().unwrap_err()
             } else {
                 // TODO: send a better error message saying some inputs in the corpus cannot be read
                 // TODO: there should be an option to ignore invalid values
-                println!("A value in the input corpus is invalid.");
-                Ok(())
+                panic!("A value in the input corpus is invalid.");
             }
         }
         FuzzerCommand::Read { input_file } => {
             // no signal handlers are installed, but that should be ok as the exit code won't be 0
-            let mut world = World::<_, ()>::new(serializer, args.clone());
-            let value = world.read_input_file(input_file)?;
+            let mut world = World::new(args.clone()).expect(WORLD_NEW_ERROR);
+            let value = world.read_input_file(input_file).expect(READ_INPUT_FILE_ERROR);
+            let value = serializer.from_data(&value).expect(SERIALIZER_FROM_DATA_ERROR);
             if let Some((cache, mutation_step)) = mutator.validate_value(&value) {
                 let input = FuzzedInput::new(value, cache, mutation_step, 0);
                 let cplx = input.complexity(&mutator);
 
-                let cell = NotUnwindSafe { value: test };
-                let input_cell = NotUnwindSafe {
-                    value: input.value.borrow(),
-                };
-                let result = catch_unwind(|| (cell.value)(input_cell.value));
+                let result = catch_unwind(AssertUnwindSafe(
+                    #[no_coverage]
+                    || (test)(input.value.borrow()),
+                ));
 
                 if result.is_err() || !result.unwrap() {
                     world.report_event::<EmptyStats>(FuzzerEvent::TestFailure, None);
-                    world.save_artifact(&input.value, cplx)?;
+                    let content = serializer.to_data(&input.value);
+                    world
+                        .save_artifact(content, cplx, serializer.extension())
+                        .expect(SAVE_ARTIFACTS_ERROR);
                     // in this case we really want to exit with a non-zero termination status here
                     // because the Read command is only used by the input minify command from cargo-fuzzcheck
                     // which checks that a crash happens by looking at the exit code
                     // so we don't want to handle any error
                     exit(TerminationStatus::TestFailure as i32);
+                } else {
+                    exit(TerminationStatus::Success as i32);
                 }
             } else {
                 // TODO: send a better error message saying some inputs in the corpus cannot be read
-                println!("A value in the input corpus is invalid.");
+                panic!("A value in the input corpus is invalid.");
             }
-            Ok(())
-        }
-        FuzzerCommand::MinifyCorpus { corpus_size } => {
-            let mut fuzzer = Fuzzer::<_, _, _, _, _, _, _>::new(
-                test,
-                mutator,
-                sensor,
-                pool,
-                args.clone(),
-                World::new(serializer, args.clone()),
-            );
-            // fuzzer.sensor_and_pool.update
-            unsafe { fuzzer.state.set_up_signal_handler() };
-
-            fuzzer.corpus_minifying_loop(*corpus_size)
         }
     };
-
     let _ = std::panic::take_hook();
 
-    result
+    let found_test_failure =
+        unsafe { matches!(reason_for_stopping, ReasonForStopping::TestFailure(_)) || DID_FIND_ANY_TEST_FAILURE };
+
+    FuzzingResult {
+        found_test_failure,
+        reason_for_stopping,
+    }
 }
