@@ -1,12 +1,27 @@
 use crate::Mutator;
 use std::{cmp::Ordering, marker::PhantomData};
 
+/**
+A mutator that wraps multiple different mutators of the same type.
+
+```
+use fuzzcheck::mutators::alternation::AlternationMutator;
+use fuzzcheck::mutators::integer_within_range::U8WithinRangeMutator;
+
+let m1 = U8WithinRangeMutator::new(3 ..= 10);
+let m2 = U8WithinRangeMutator::new(78 ..= 200);
+
+let m = AlternationMutator::new(vec![m1, m2]);
+
+// m will produce values either in 3..=10 or in 78..=200
+```
+*/
 pub struct AlternationMutator<T, M>
 where
     T: Clone,
     M: Mutator<T>,
 {
-    pub mutators: Vec<M>,
+    mutators: Vec<M>,
     complexity_from_choice: f64,
     max_complexity: f64,
     min_complexity: f64,
@@ -22,15 +37,43 @@ where
     #[no_coverage]
     pub fn new(mutators: Vec<M>) -> Self {
         assert!(!mutators.is_empty());
+        let complexity_from_choice = crate::mutators::size_to_cplxity(mutators.len());
+
         let max_complexity = mutators
             .iter()
-            .map(|m| m.max_complexity())
-            .max_by(|x, y| x.partial_cmp(y).unwrap_or(Ordering::Equal))
+            .map(
+                #[no_coverage]
+                |m| {
+                    let max_cplx = m.max_complexity();
+                    if max_cplx == 0. {
+                        complexity_from_choice
+                    } else {
+                        max_cplx
+                    }
+                },
+            )
+            .max_by(
+                #[no_coverage]
+                |x, y| x.partial_cmp(y).unwrap_or(Ordering::Equal),
+            )
             .unwrap();
         let min_complexity = mutators
             .iter()
-            .map(|m| m.max_complexity())
-            .min_by(|x, y| x.partial_cmp(y).unwrap_or(Ordering::Equal))
+            .map(
+                #[no_coverage]
+                |m| {
+                    let min_cplx = m.min_complexity();
+                    if min_cplx == 0. {
+                        complexity_from_choice
+                    } else {
+                        min_cplx
+                    }
+                },
+            )
+            .min_by(
+                #[no_coverage]
+                |x, y| x.partial_cmp(y).unwrap_or(Ordering::Equal),
+            )
             .unwrap();
         let complexity_from_choice = crate::mutators::size_to_cplxity(mutators.len());
         Self {
@@ -44,25 +87,38 @@ where
     }
 }
 
-#[derive(Clone)]
+#[doc(hidden)]
+#[derive(Clone, Debug, PartialEq)]
 pub struct ArbitraryStep<AS> {
     inner: Vec<AS>,
     indices: Vec<usize>,
     idx: usize,
 }
 
-#[derive(Clone)]
+#[doc(hidden)]
+#[derive(Clone, Debug, PartialEq)]
+pub struct RecursingPartIndex<RPI> {
+    inner: Vec<RPI>,
+    indices: Vec<usize>,
+}
+
+#[doc(hidden)]
+#[derive(Clone, Debug, PartialEq)]
 pub struct MutationStep<MS, AS> {
+    step: usize,
+    mutator_idx: usize,
     inner: MS,
     arbitrary: AS,
 }
 
+#[doc(hidden)]
 #[derive(Clone, Debug, PartialEq)]
 pub struct Cache<C> {
     inner: C,
     mutator_idx: usize,
 }
 
+#[doc(hidden)]
 pub enum UnmutateToken<T, U> {
     Replace(T),
     Inner(usize, U),
@@ -74,8 +130,14 @@ where
     M: Mutator<T>,
 {
     #[no_coverage]
-    fn default_mutation_step(&self, inner: M::MutationStep, idx: usize) -> <Self as Mutator<T>>::MutationStep {
+    fn default_mutation_step(
+        &self,
+        inner: M::MutationStep,
+        idx: usize,
+    ) -> MutationStep<M::MutationStep, <Self as Mutator<T>>::ArbitraryStep> {
         MutationStep {
+            step: 0,
+            mutator_idx: idx,
             inner,
             arbitrary: {
                 let mut step = self.default_arbitrary_step();
@@ -92,20 +154,29 @@ where
 {
     #[no_coverage]
     fn complexity_from_inner(&self, cplx: f64) -> f64 {
-        self.complexity_from_choice + cplx
+        if cplx == 0. {
+            self.complexity_from_choice
+        } else {
+            cplx
+        }
     }
 }
 
 impl<T, M> Mutator<T> for AlternationMutator<T, M>
 where
-    T: Clone,
+    T: Clone + 'static,
     M: Mutator<T>,
 {
-    type Cache = Cache<M::Cache>;
-    type MutationStep = MutationStep<M::MutationStep, Self::ArbitraryStep>;
+    #[doc(hidden)]
+    type Cache = Vec<Cache<M::Cache>>;
+    #[doc(hidden)]
+    type MutationStep = Vec<MutationStep<M::MutationStep, Self::ArbitraryStep>>;
+    #[doc(hidden)]
     type ArbitraryStep = ArbitraryStep<M::ArbitraryStep>;
+    #[doc(hidden)]
     type UnmutateToken = UnmutateToken<T, M::UnmutateToken>;
 
+    #[doc(hidden)]
     #[no_coverage]
     fn default_arbitrary_step(&self) -> Self::ArbitraryStep {
         Self::ArbitraryStep {
@@ -122,37 +193,47 @@ where
         }
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn validate_value(&self, value: &T) -> Option<(Self::Cache, Self::MutationStep)> {
+        let mut caches = vec![];
+        let mut steps = vec![];
         for (idx, mutator) in self.mutators.iter().enumerate() {
             if let Some((c, s)) = mutator.validate_value(value) {
-                return Some((
-                    Cache {
-                        inner: c,
-                        mutator_idx: idx,
-                    },
-                    self.default_mutation_step(s, idx),
-                ));
+                caches.push(Cache {
+                    inner: c,
+                    mutator_idx: idx,
+                });
+                steps.push(self.default_mutation_step(s, idx));
             }
         }
-        None
+        if caches.is_empty() {
+            return None;
+        } else {
+            Some((caches, steps))
+        }
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn max_complexity(&self) -> f64 {
         self.complexity_from_inner(self.max_complexity)
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn min_complexity(&self) -> f64 {
         self.complexity_from_inner(self.min_complexity)
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn complexity(&self, value: &T, cache: &Self::Cache) -> f64 {
+        let cache = &cache[0];
         self.complexity_from_inner(self.mutators[cache.mutator_idx].complexity(value, &cache.inner))
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn ordered_arbitrary(&self, step: &mut Self::ArbitraryStep, max_cplx: f64) -> Option<(T, f64)> {
         if step.indices.is_empty() {
@@ -161,7 +242,6 @@ where
         if max_cplx < self.min_complexity() {
             return None;
         }
-        let max_cplx = max_cplx - self.complexity_from_choice;
 
         let idx = step.indices[step.idx % step.indices.len()];
         let mutator = &self.mutators[idx];
@@ -175,16 +255,17 @@ where
         }
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn random_arbitrary(&self, max_cplx: f64) -> (T, f64) {
         let idx = self.rng.usize(..self.mutators.len());
         let mutator = &self.mutators[idx];
-        let max_cplx = max_cplx - self.complexity_from_choice;
 
         let (v, c) = mutator.random_arbitrary(max_cplx);
         (v, self.complexity_from_inner(c))
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn ordered_mutate(
         &self,
@@ -196,7 +277,9 @@ where
         if max_cplx < self.min_complexity() {
             return None;
         }
-        let max_cplx = max_cplx - self.complexity_from_choice;
+        if step.is_empty() {
+            return None;
+        }
 
         if self.rng.usize(..100) == 0 {
             let (new_value, cplx) = self.random_arbitrary(max_cplx);
@@ -204,23 +287,50 @@ where
             return Some((UnmutateToken::Replace(old_value), cplx));
         }
 
-        let idx = cache.mutator_idx;
+        let step_idx = self.rng.usize(..step.len());
+        let chosen_step = &mut step[step_idx];
+        chosen_step.step += 1;
+        if chosen_step.step < 20 {
+            if let Some((mut v, cplx)) = self.ordered_arbitrary(&mut chosen_step.arbitrary, max_cplx) {
+                std::mem::swap(value, &mut v);
+                return Some((UnmutateToken::Replace(v), cplx));
+            }
+        }
+
+        let mutator_idx = chosen_step.mutator_idx;
+        let chosen_cache = cache.iter_mut().find(|c| c.mutator_idx == mutator_idx).unwrap();
+
+        let idx = chosen_cache.mutator_idx;
+        assert_eq!(idx, mutator_idx);
+
         let mutator = &self.mutators[idx];
-        if let Some((t, cplx)) = mutator.ordered_mutate(value, &mut cache.inner, &mut step.inner, max_cplx) {
+        if let Some((t, cplx)) =
+            mutator.ordered_mutate(value, &mut chosen_cache.inner, &mut chosen_step.inner, max_cplx)
+        {
             Some((UnmutateToken::Inner(idx, t), self.complexity_from_inner(cplx)))
-        } else if let Some((mut v, cplx)) = self.ordered_arbitrary(&mut step.arbitrary, max_cplx) {
-            std::mem::swap(value, &mut v);
-            Some((UnmutateToken::Replace(v), cplx))
         } else {
-            None
+            if let Some((mut v, cplx)) = self.ordered_arbitrary(&mut chosen_step.arbitrary, max_cplx) {
+                std::mem::swap(value, &mut v);
+                Some((UnmutateToken::Replace(v), cplx))
+            } else {
+                step.remove(step_idx);
+                if step.is_empty() {
+                    None
+                } else {
+                    self.ordered_mutate(value, cache, step, max_cplx)
+                }
+            }
         }
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn random_mutate(&self, value: &mut T, cache: &mut Self::Cache, max_cplx: f64) -> (Self::UnmutateToken, f64) {
+        let cache_idx = self.rng.usize(..cache.len());
+        let cache = &mut cache[cache_idx];
+
         let idx = cache.mutator_idx;
         let mutator = &self.mutators[idx];
-        let max_cplx = max_cplx - self.complexity_from_choice;
         // this ensures that `arbitrary` is used instead of a unit mutator that will return
         // the same thing every time
         // there should be a better way to prevent this though
@@ -235,6 +345,7 @@ where
         (UnmutateToken::Inner(idx, t), self.complexity_from_inner(cplx))
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn unmutate(&self, value: &mut T, cache: &mut Self::Cache, t: Self::UnmutateToken) {
         match t {
@@ -243,8 +354,54 @@ where
             }
             UnmutateToken::Inner(idx, t) => {
                 let mutator = &self.mutators[idx];
+                let cache = cache.iter_mut().find(|c| c.mutator_idx == idx).unwrap();
                 mutator.unmutate(value, &mut cache.inner, t);
             }
         }
     }
+
+    #[doc(hidden)]
+    type RecursingPartIndex = RecursingPartIndex<M::RecursingPartIndex>;
+
+    #[doc(hidden)]
+    #[no_coverage]
+    fn default_recursing_part_index(&self, value: &T, cache: &Self::Cache) -> Self::RecursingPartIndex {
+        Self::RecursingPartIndex {
+            inner: cache
+                .iter()
+                .map(|c| self.mutators[c.mutator_idx].default_recursing_part_index(value, &c.inner))
+                .collect(),
+            indices: cache.iter().map(|c| c.mutator_idx).collect(),
+        }
+    }
+
+    #[doc(hidden)]
+    #[no_coverage]
+    fn recursing_part<'a, V, N>(&self, parent: &N, value: &'a T, index: &mut Self::RecursingPartIndex) -> Option<&'a V>
+    where
+        V: Clone + 'static,
+        N: Mutator<V>,
+    {
+        // if vector is empty, return Nlne
+        if index.inner.is_empty() {
+            return None;
+        }
+
+        // choose a recursing part randomly
+        let choice = self.rng.usize(..index.inner.len());
+        let subindex = &mut index.inner[choice];
+        let mutator_index = index.indices[choice];
+
+        // call it
+        let result = self.mutators[mutator_index].recursing_part::<V, N>(parent, value, subindex);
+
+        // if none, remove it from the vector and call the function again
+        if result.is_none() {
+            index.inner.remove(choice);
+            index.indices.remove(choice);
+            self.recursing_part::<V, N>(parent, value, index)
+        } else {
+            result
+        }
+    }
 }
