@@ -1,5 +1,16 @@
 use crate::Mutator;
 
+/** Wrap a mutator and prioritise the generation of a few given values.
+```
+use fuzzcheck::DefaultMutator;
+use fuzzcheck::mutators::dictionary::DictionaryMutator;
+
+let m = usize::default_mutator();
+let m = DictionaryMutator::new(m, [256, 65_536, 1_000_000]);
+// m will first generate the values given to the DictionaryMutator constructor
+// and will then use usize’s default mutator
+```
+*/
 pub struct DictionaryMutator<T: Clone, M: Mutator<T>> {
     m: M,
     dictionary: Vec<(T, f64)>,
@@ -7,8 +18,9 @@ pub struct DictionaryMutator<T: Clone, M: Mutator<T>> {
 }
 impl<T: Clone, M: Mutator<T>> DictionaryMutator<T, M> {
     #[no_coverage]
-    pub fn new(value_mutator: M, dictionary: impl Iterator<Item = T>) -> Self {
+    pub fn new(value_mutator: M, dictionary: impl IntoIterator<Item = T>) -> Self {
         let dictionary = dictionary
+            .into_iter()
             .filter_map(|v| {
                 if let Some((cache, _)) = value_mutator.validate_value(&v) {
                     let complexity = value_mutator.complexity(&v, &cache);
@@ -55,17 +67,23 @@ impl<T> Default for ArbitraryStep<T> {
     }
 }
 
-impl<T: Clone, M: Mutator<T>> Mutator<T> for DictionaryMutator<T, M> {
+impl<T: Clone + 'static, M: Mutator<T>> Mutator<T> for DictionaryMutator<T, M> {
+    #[doc(hidden)]
     type Cache = M::Cache;
+    #[doc(hidden)]
     type MutationStep = self::MutationStep<M::MutationStep>;
+    #[doc(hidden)]
     type ArbitraryStep = self::ArbitraryStep<M::ArbitraryStep>;
+    #[doc(hidden)]
     type UnmutateToken = UnmutateToken<T, M>;
 
+    #[doc(hidden)]
     #[no_coverage]
     fn default_arbitrary_step(&self) -> Self::ArbitraryStep {
         <_>::default()
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn validate_value(&self, value: &T) -> Option<(Self::Cache, Self::MutationStep)> {
         if let Some((cache, step)) = self.m.validate_value(value) {
@@ -75,6 +93,7 @@ impl<T: Clone, M: Mutator<T>> Mutator<T> for DictionaryMutator<T, M> {
         }
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn ordered_arbitrary(&self, step: &mut Self::ArbitraryStep, max_cplx: f64) -> Option<(T, f64)> {
         match step {
@@ -96,6 +115,7 @@ impl<T: Clone, M: Mutator<T>> Mutator<T> for DictionaryMutator<T, M> {
         }
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn random_arbitrary(&self, max_cplx: f64) -> (T, f64) {
         let (v, c) = if !self.dictionary.is_empty() && self.rng.usize(..20) == 0 {
@@ -107,21 +127,25 @@ impl<T: Clone, M: Mutator<T>> Mutator<T> for DictionaryMutator<T, M> {
         (v, c)
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn max_complexity(&self) -> f64 {
         self.m.max_complexity()
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn min_complexity(&self) -> f64 {
         self.m.min_complexity()
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn complexity(&self, value: &T, cache: &Self::Cache) -> f64 {
         self.m.complexity(value, cache)
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn ordered_mutate(
         &self,
@@ -144,6 +168,7 @@ impl<T: Clone, M: Mutator<T>> Mutator<T> for DictionaryMutator<T, M> {
         }
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn random_mutate(&self, value: &mut T, cache: &mut Self::Cache, max_cplx: f64) -> (Self::UnmutateToken, f64) {
         if !self.dictionary.is_empty() && self.rng.usize(..20) == 0 {
@@ -159,6 +184,7 @@ impl<T: Clone, M: Mutator<T>> Mutator<T> for DictionaryMutator<T, M> {
         }
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn unmutate(&self, value: &mut T, cache: &mut Self::Cache, t: Self::UnmutateToken) {
         match t {
@@ -168,4 +194,20 @@ impl<T: Clone, M: Mutator<T>> Mutator<T> for DictionaryMutator<T, M> {
             UnmutateToken::Unmutate(t) => self.m.unmutate(value, cache, t),
         }
     }
+    #[doc(hidden)]
+    type RecursingPartIndex = M::RecursingPartIndex;
+    #[doc(hidden)]
+    #[no_coverage]
+    fn default_recursing_part_index(&self, value: &T, cache: &Self::Cache) -> Self::RecursingPartIndex {
+        self.m.default_recursing_part_index(value, cache)
+    }
+    #[doc(hidden)]
+    #[no_coverage]
+    fn recursing_part<'a, V, N>(&self, parent: &N, value: &'a T, index: &mut Self::RecursingPartIndex) -> Option<&'a V>
+    where
+        V: Clone + 'static,
+        N: Mutator<V>,
+    {
+        self.m.recursing_part::<V, N>(parent, value, index)
+    }
 }
