@@ -11,11 +11,16 @@ where
     M1: Mutator<T>,
     M2: Mutator<T>,
 {
+    #[doc(hidden)]
     type Cache = Either<M1::Cache, M2::Cache>;
+    #[doc(hidden)]
     type MutationStep = Either<M1::MutationStep, M2::MutationStep>;
+    #[doc(hidden)]
     type ArbitraryStep = Either<M1::ArbitraryStep, M2::ArbitraryStep>;
+    #[doc(hidden)]
     type UnmutateToken = Either<M1::UnmutateToken, M2::UnmutateToken>;
 
+    #[doc(hidden)]
     #[no_coverage]
     fn default_arbitrary_step(&self) -> Self::ArbitraryStep {
         match self {
@@ -24,6 +29,7 @@ where
         }
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn validate_value(&self, value: &T) -> Option<(Self::Cache, Self::MutationStep)> {
         match self {
@@ -38,6 +44,7 @@ where
         }
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn max_complexity(&self) -> f64 {
         match self {
@@ -46,6 +53,7 @@ where
         }
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn min_complexity(&self) -> f64 {
         match self {
@@ -54,6 +62,7 @@ where
         }
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn complexity(&self, value: &T, cache: &Self::Cache) -> f64 {
         match (self, cache) {
@@ -63,6 +72,7 @@ where
         }
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn ordered_arbitrary(&self, step: &mut Self::ArbitraryStep, max_cplx: f64) -> Option<(T, f64)> {
         match (self, step) {
@@ -72,6 +82,7 @@ where
         }
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn random_arbitrary(&self, max_cplx: f64) -> (T, f64) {
         match self {
@@ -80,6 +91,7 @@ where
         }
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn ordered_mutate(
         &self,
@@ -101,6 +113,7 @@ where
         }
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn random_mutate(&self, value: &mut T, cache: &mut Self::Cache, max_cplx: f64) -> (Self::UnmutateToken, f64) {
         match (self, cache) {
@@ -116,6 +129,7 @@ where
         }
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn unmutate(&self, value: &mut T, cache: &mut Self::Cache, t: Self::UnmutateToken) {
         match (self, cache, t) {
@@ -128,4 +142,29 @@ where
             _ => panic!(),
         }
     }
+
+    #[doc(hidden)]
+    type RecursingPartIndex = Either<M1::RecursingPartIndex, M2::RecursingPartIndex>;
+    #[doc(hidden)]
+    #[no_coverage]
+    fn default_recursing_part_index(&self, value: &T, cache: &Self::Cache) -> Self::RecursingPartIndex {
+        match (self, cache) {
+            (Either::Left(m), Either::Left(c)) => Either::Left(m.default_recursing_part_index(value, c)),
+            (Either::Right(m), Either::Right(c)) => Either::Right(m.default_recursing_part_index(value, c)),
+            _ => unreachable!(),
+        }
+    }
+    #[doc(hidden)]
+    #[no_coverage]
+    fn recursing_part<'a, V, N>(&self, parent: &N, value: &'a T, index: &mut Self::RecursingPartIndex) -> Option<&'a V>
+    where
+        V: Clone + 'static,
+        N: Mutator<V>,
+    {
+        match (self, index) {
+            (Either::Left(m), Either::Left(i)) => m.recursing_part::<V, N>(parent, value, i),
+            (Either::Right(m), Either::Right(i)) => m.recursing_part::<V, N>(parent, value, i),
+            _ => panic!(),
+        }
+    }
 }
