@@ -1,8 +1,8 @@
 use crate::mutators::either::Either;
-use crate::mutators::recursive::{RecurToMutator, RecursiveMutator};
+use crate::mutators::recursive::RecurToMutator;
 use crate::Mutator;
+use std::fmt::Debug;
 use std::marker::PhantomData;
-
 /**
     A type that that can propagate updates to a value (From) to
     another value (To) based on a Token.
@@ -10,7 +10,7 @@ use std::marker::PhantomData;
     The token should be the UnmutateToken of a mutator.
 */
 #[doc(hidden)] // hide the documentation because it pollutes the documentation of all other types
-pub trait IncrementalMapping<From: Clone, To, M: Mutator<From>> {
+pub trait IncrementalMapping<From: Clone + 'static, To: Clone + 'static, M: Mutator<From>> {
     /// The `from_value` was already mutated. The `token` encodes how to unmutate `from_value`. Based on `token` and `from_value`,
     /// this function updates the `to_value`.
     fn mutate_value_from_token(&mut self, from_value: &From, to_value: &mut To, token: &M::UnmutateToken);
@@ -21,10 +21,11 @@ pub trait IncrementalMapping<From: Clone, To, M: Mutator<From>> {
 
 pub struct Cache<From, To, M, Map>
 where
-    From: Clone,
+    From: Clone + 'static,
     To: Clone + std::convert::From<From>,
     M: Mutator<From>,
     Map: IncrementalMapping<From, To, M> + for<'a> std::convert::From<&'a From>,
+    Self: 'static,
 {
     from_value: From,
     from_cache: M::Cache,
@@ -33,10 +34,11 @@ where
 }
 impl<From, To, M, Map> Clone for Cache<From, To, M, Map>
 where
-    From: Clone,
+    From: Clone + 'static,
     To: Clone + std::convert::From<From>,
     M: Mutator<From>,
     Map: IncrementalMapping<From, To, M> + for<'a> std::convert::From<&'a From> + Clone,
+    Self: 'static,
 {
     fn clone(&self) -> Self {
         Self {
@@ -47,14 +49,49 @@ where
         }
     }
 }
+impl<From, To, M, Map> Debug for Cache<From, To, M, Map>
+where
+    From: Clone + Debug + 'static,
+    To: Clone + Debug + std::convert::From<From>,
+    M::Cache: Debug,
+    M: Mutator<From>,
+    Map: IncrementalMapping<From, To, M> + for<'a> std::convert::From<&'a From> + Debug,
+    Self: 'static,
+{
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        f.debug_struct("Cache")
+            .field("from_value", &self.from_value)
+            .field("from_cache", &self.from_cache)
+            .field("map", &self.map)
+            .field("_phantom", &self._phantom)
+            .finish()
+    }
+}
+impl<From, To, M, Map> PartialEq for Cache<From, To, M, Map>
+where
+    From: Clone + PartialEq + 'static,
+    To: Clone + PartialEq + std::convert::From<From>,
+    M: Mutator<From>,
+    M::Cache: PartialEq,
+    Map: IncrementalMapping<From, To, M> + for<'a> std::convert::From<&'a From> + PartialEq,
+    Self: 'static,
+{
+    fn eq(&self, other: &Self) -> bool {
+        self.from_value == other.from_value
+            && self.from_cache == other.from_cache
+            && self.map == other.map
+            && self._phantom == other._phantom
+    }
+}
 
 pub struct IncrementalMapMutator<From, To, M, Map, Parse>
 where
-    From: Clone,
+    From: Clone + 'static,
     To: Clone + std::convert::From<From>,
     M: Mutator<From>,
     Map: IncrementalMapping<From, To, M> + for<'a> std::convert::From<&'a From>,
     Parse: Fn(&To) -> Option<From>,
+    Self: 'static,
 {
     parse: Parse,
     mutator: M,
@@ -67,6 +104,7 @@ where
     M: Mutator<From>,
     Map: IncrementalMapping<From, To, M> + for<'a> std::convert::From<&'a From>,
     Parse: Fn(&To) -> Option<From>,
+    Self: 'static,
 {
     #[no_coverage]
     pub fn new(parse: Parse, mutator: M) -> Self {
@@ -80,22 +118,29 @@ where
 
 impl<From, To, M, Map, Parse> Mutator<To> for IncrementalMapMutator<From, To, M, Map, Parse>
 where
-    From: Clone,
-    To: Clone + std::convert::From<From>,
+    From: Clone + 'static,
+    To: Clone + std::convert::From<From> + 'static,
     M: Mutator<From>,
     Map: IncrementalMapping<From, To, M> + for<'a> std::convert::From<&'a From> + Clone,
     Parse: Fn(&To) -> Option<From>,
+    Self: 'static,
 {
+    #[doc(hidden)]
     type Cache = Cache<From, To, M, Map>;
+    #[doc(hidden)]
     type MutationStep = M::MutationStep;
+    #[doc(hidden)]
     type ArbitraryStep = M::ArbitraryStep;
+    #[doc(hidden)]
     type UnmutateToken = M::UnmutateToken;
 
+    #[doc(hidden)]
     #[no_coverage]
     fn default_arbitrary_step(&self) -> Self::ArbitraryStep {
         self.mutator.default_arbitrary_step()
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn validate_value(&self, value: &To) -> Option<(Self::Cache, Self::MutationStep)> {
         let from_value = (self.parse)(value)?;
@@ -110,21 +155,25 @@ where
         Some((cache, mutation_step))
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn max_complexity(&self) -> f64 {
         self.mutator.max_complexity()
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn min_complexity(&self) -> f64 {
         self.mutator.min_complexity()
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn complexity(&self, _value: &To, cache: &Self::Cache) -> f64 {
         self.mutator.complexity(&cache.from_value, &cache.from_cache)
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn ordered_arbitrary(&self, step: &mut Self::ArbitraryStep, max_cplx: f64) -> Option<(To, f64)> {
         let (value, cplx) = self.mutator.ordered_arbitrary(step, max_cplx)?;
@@ -132,6 +181,7 @@ where
         Some((x, cplx))
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn random_arbitrary(&self, max_cplx: f64) -> (To, f64) {
         let (value, cplx) = self.mutator.random_arbitrary(max_cplx);
@@ -139,6 +189,7 @@ where
         (x, cplx)
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn ordered_mutate(
         &self,
@@ -154,6 +205,7 @@ where
         Some((token, cplx))
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn random_mutate(&self, value: &mut To, cache: &mut Self::Cache, max_cplx: f64) -> (Self::UnmutateToken, f64) {
         let (token, cplx) = self
@@ -163,44 +215,79 @@ where
         (token, cplx)
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn unmutate(&self, value: &mut To, cache: &mut Self::Cache, t: Self::UnmutateToken) {
         cache.map.unmutate_value_from_token(value, &t);
         self.mutator.unmutate(&mut cache.from_value, &mut cache.from_cache, t);
     }
-}
+    // TODO: Not yet supported for MapMutator
+    // it would require `recursing_part` to take the `cache` value as argument as well
+    // maybe I should do that, but I haven't thought about it yet
+    type RecursingPartIndex = ();
 
-impl<From, To, M, Map> IncrementalMapping<From, To, RecursiveMutator<M>> for Map
-where
-    From: Clone,
-    To: Clone,
-    M: Mutator<From>,
-    Self: IncrementalMapping<From, To, M>,
-{
-    #[no_coverage]
-    fn mutate_value_from_token(
-        &mut self,
-        from_value: &From,
-        to_value: &mut To,
-        token: &<RecursiveMutator<M> as Mutator<From>>::UnmutateToken,
-    ) {
-        <Self as IncrementalMapping<From, To, M>>::mutate_value_from_token(self, from_value, to_value, token);
-    }
+    fn default_recursing_part_index(&self, _value: &To, _cache: &Self::Cache) -> Self::RecursingPartIndex {}
 
-    #[no_coverage]
-    fn unmutate_value_from_token(
-        &mut self,
-        to_value: &mut To,
-        token: &<RecursiveMutator<M> as Mutator<From>>::UnmutateToken,
-    ) {
-        <Self as IncrementalMapping<From, To, M>>::unmutate_value_from_token(self, to_value, token);
+    fn recursing_part<'a, V, N>(
+        &self,
+        _parent: &N,
+        _value: &'a To,
+        _index: &mut Self::RecursingPartIndex,
+    ) -> Option<&'a V>
+    where
+        V: Clone + 'static,
+        N: Mutator<V>,
+    {
+        None
     }
 }
 
+// impl<From, To, M, Map> IncrementalMapping<From, To, RecursiveMutator<M>> for Map
+// where
+//     From: Clone + 'static,
+//     To: Clone + 'static,
+//     M: Mutator<From>,
+//     Self: IncrementalMapping<From, To, M>,
+// {
+//     #[no_coverage]
+//     fn mutate_value_from_token(
+//         &mut self,
+//         from_value: &From,
+//         to_value: &mut To,
+//         token: &<RecursiveMutator<M> as Mutator<From>>::UnmutateToken,
+//     ) {
+//         match token {
+//             RecursiveMutatorUnmutateToken::Replace(v) => {
+//                 *to_value = todo!();
+//             }
+//             RecursiveMutatorUnmutateToken::Token(t) => {
+//                 <Self as IncrementalMapping<From, To, M>>::mutate_value_from_token(self, from_value, to_value, t);
+//             }
+//         }
+//     }
+
+//     #[doc(hidden)]
+//     #[no_coverage]
+//     fn unmutate_value_from_token(
+//         &mut self,
+//         to_value: &mut To,
+//         token: &<RecursiveMutator<M> as Mutator<From>>::UnmutateToken,
+//     ) {
+//         match token {
+//             RecursiveMutatorUnmutateToken::Replace(v) => {
+//                 *to_value = todo!();
+//             }
+//             RecursiveMutatorUnmutateToken::Token(t) => {
+//                 <Self as IncrementalMapping<From, To, M>>::unmutate_value_from_token(self, to_value, t);
+//             }
+//         }
+//     }
+// }
+
 impl<From, To, Map, M1, M2> IncrementalMapping<From, To, Either<M1, M2>> for Map
 where
-    From: Clone,
-    To: Clone,
+    From: Clone + 'static,
+    To: Clone + 'static,
     M1: Mutator<From>,
     M2: Mutator<From>,
     Self: IncrementalMapping<From, To, M1> + IncrementalMapping<From, To, M2>,
@@ -222,6 +309,7 @@ where
         }
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn unmutate_value_from_token(
         &mut self,
@@ -241,8 +329,8 @@ where
 
 impl<From, To, M, Map> IncrementalMapping<From, To, RecurToMutator<M>> for Map
 where
-    From: Clone,
-    To: Clone,
+    From: Clone + 'static,
+    To: Clone + 'static,
     M: Mutator<From>,
     Self: IncrementalMapping<From, To, M>,
 {
@@ -256,6 +344,7 @@ where
         <Self as IncrementalMapping<From, To, M>>::mutate_value_from_token(self, from_value, to_value, token);
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn unmutate_value_from_token(
         &mut self,
