@@ -12,10 +12,8 @@ use crate::Mutator;
 
     One idea to create arbitrary integers that don't repeat themselves and span the whole search space was
     to use a binary-search-like approach, as written in the function binary_search_arbitrary. However that
-    turns out to be quite slow for an integer mutator. So what I do instead is create a sequence of 256 integers
-    using that approach, which I store in a variable called “shuffled integers”. So shuffled integers is a
-    vector that look something like this:
-        [0, 256, 128, 192, 64, 224, 32, ...]
+    turns out to be quite slow for an integer mutator. So what I do instead is create a sequence of 256
+    random non-repeating integers, which I store in a variable called “shuffled integers”
     Now, for an 8-bit integer type, it is enough to simply index that vector to get an arbitrary value that
     respects all the criteria I laid above. But for types that have 16, 32, 64, or 128 bits, I can't do that.
     So I index the shuffled_integers vector multiple times until I have all the bits I need. For an u32, I need
@@ -85,11 +83,11 @@ macro_rules! impl_int_mutator {
             #[no_coverage]
             fn default() -> Self {
                 let mut shuffled_integers = [0; 256];
-                let rng = fastrand::Rng::default();
-                let xor = rng.u8(..);
-                for (i, x) in shuffled_integers.iter_mut().enumerate() {
-                    *x = binary_search_arbitrary_u8(0, u8::MAX, i as u64) ^ xor;
+                for i in 0..=255_u8 {
+                    shuffled_integers[i as usize] = i;
                 }
+                let rng = fastrand::Rng::default();
+                rng.shuffle(&mut shuffled_integers);
                 $name_mutator {
                     shuffled_integers,
                     rng,
@@ -152,39 +150,43 @@ macro_rules! impl_int_mutator {
         }
 
         impl Mutator<$name> for $name_mutator {
+            #[doc(hidden)]
             type Cache = ();
+            #[doc(hidden)]
             type MutationStep = u64; // mutation step
+            #[doc(hidden)]
             type ArbitraryStep = u64;
+            #[doc(hidden)]
             type UnmutateToken = $name; // old value
-
+            #[doc(hidden)]
             #[no_coverage]
             fn default_arbitrary_step(&self) -> Self::ArbitraryStep {
                 <_>::default()
             }
-
+            #[doc(hidden)]
             #[no_coverage]
             fn validate_value(&self, _value: &$name) -> Option<(Self::Cache, Self::MutationStep)> {
                 Some(((), INITIAL_MUTATION_STEP))
             }
 
             /// The maximum complexity of an input of this type
-
+            #[doc(hidden)]
             #[no_coverage]
             fn max_complexity(&self) -> f64 {
                 <$name>::BITS as f64
             }
             /// The minimum complexity of an input of this type
-
+            #[doc(hidden)]
             #[no_coverage]
             fn min_complexity(&self) -> f64 {
                 <$name>::BITS as f64
             }
-
+            #[doc(hidden)]
             #[no_coverage]
             fn complexity(&self, _value: &$name, _cache: &Self::Cache) -> f64 {
                 <$name>::BITS as f64
             }
-
+            #[doc(hidden)]
             #[no_coverage]
             fn ordered_arbitrary(&self, step: &mut Self::ArbitraryStep, max_cplx: f64) -> Option<($name, f64)> {
                 if max_cplx < self.min_complexity() {
@@ -198,13 +200,13 @@ macro_rules! impl_int_mutator {
                     Some((value, <$name>::BITS as f64))
                 }
             }
-
+            #[doc(hidden)]
             #[no_coverage]
             fn random_arbitrary(&self, _max_cplx: f64) -> ($name, f64) {
                 let value = self.rng.$name(..);
                 (value, <$name>::BITS as f64)
             }
-
+            #[doc(hidden)]
             #[no_coverage]
             fn ordered_mutate(
                 &self,
@@ -238,7 +240,7 @@ macro_rules! impl_int_mutator {
 
                 Some((token, <$name>::BITS as f64))
             }
-
+            #[doc(hidden)]
             #[no_coverage]
             fn random_mutate(
                 &self,
@@ -248,11 +250,31 @@ macro_rules! impl_int_mutator {
             ) -> (Self::UnmutateToken, f64) {
                 (std::mem::replace(value, self.rng.$name(..)), <$name>::BITS as f64)
             }
-
+            #[doc(hidden)]
             #[no_coverage]
             fn unmutate(&self, value: &mut $name, _cache: &mut Self::Cache, t: Self::UnmutateToken) {
                 *value = t;
             }
+
+            #[doc(hidden)]
+            type RecursingPartIndex = ();
+            #[doc(hidden)]
+            #[no_coverage]
+            fn default_recursing_part_index(&self, _value: &$name, _cache: &Self::Cache) -> Self::RecursingPartIndex {}
+            #[doc(hidden)]
+            #[no_coverage]
+            fn recursing_part<'a, V, N>(
+                &self,
+                _parent: &N,
+                _value: &'a $name,
+                _index: &mut Self::RecursingPartIndex,
+            ) -> Option<&'a V>
+            where
+                V: Clone + 'static,
+                N: Mutator<V> + 'static,
+            {
+                None
+            }
         }
 
         impl DefaultMutator for $name {
