@@ -33,7 +33,14 @@ macro_rules! impl_int_mutator_constrained {
                     }
                     Bound::Unbounded => <$name>::MAX,
                 };
-                assert!(start <= end);
+                if !(start <= end) {
+                    panic!(
+                        "You have provided a character range where the value of the start of the range \
+                        is larger than the end of the range!\nRange start: {:#?}\nRange end: {:#?}",
+                        range.start_bound(),
+                        range.end_bound()
+                    )
+                }
                 Self {
                     start_range: start,
                     len_range: end.wrapping_sub(start) as $name_unsigned,
@@ -43,9 +50,13 @@ macro_rules! impl_int_mutator_constrained {
         }
 
         impl Mutator<$name> for $name_mutator {
+            #[doc(hidden)]
             type Cache = ();
+            #[doc(hidden)]
             type MutationStep = u64; // mutation step
+            #[doc(hidden)]
             type ArbitraryStep = u64;
+            #[doc(hidden)]
             type UnmutateToken = $name; // old value
 
             #[no_coverage]
@@ -142,6 +153,26 @@ macro_rules! impl_int_mutator_constrained {
             fn unmutate(&self, value: &mut $name, _cache: &mut Self::Cache, t: Self::UnmutateToken) {
                 *value = t;
             }
+
+            #[doc(hidden)]
+            type RecursingPartIndex = ();
+            #[doc(hidden)]
+            #[no_coverage]
+            fn default_recursing_part_index(&self, _value: &$name, _cache: &Self::Cache) -> Self::RecursingPartIndex {}
+            #[doc(hidden)]
+            #[no_coverage]
+            fn recursing_part<'a, V, N>(
+                &self,
+                _parent: &N,
+                _value: &'a $name,
+                _index: &mut Self::RecursingPartIndex,
+            ) -> Option<&'a V>
+            where
+                V: Clone + 'static,
+                N: Mutator<V>,
+            {
+                None
+            }
         }
     };
 }
