@@ -9,14 +9,14 @@ use std::{
 };
 use std::{iter::repeat, marker::PhantomData};
 
-pub struct VecMutator<T: Clone, M: Mutator<T>> {
+pub struct VecMutator<T: Clone + 'static, M: Mutator<T>> {
     pub rng: Rng,
     pub m: M,
     pub len_range: RangeInclusive<usize>,
     pub dictionary: Vec<Vec<T>>,
     _phantom: PhantomData<T>,
 }
-impl<T: Clone, M: Mutator<T>> VecMutator<T, M> {
+impl<T: Clone + 'static, M: Mutator<T>> VecMutator<T, M> {
     #[no_coverage]
     pub fn new(mutator: M, len_range: RangeInclusive<usize>) -> Self {
         Self {
@@ -53,7 +53,7 @@ where
         }
     }
 }
-impl<T: Clone> DefaultMutator for Vec<T>
+impl<T: Clone + 'static> DefaultMutator for Vec<T>
 where
     T: DefaultMutator,
 {
@@ -64,8 +64,27 @@ where
     }
 }
 
-#[derive(Clone)]
+#[derive(Clone, Debug, PartialEq)]
+pub struct ExecutedMutations {
+    make_empty: bool,      // true if it's been tried
+    remove_element: usize, // index of removed element, starts at 0 and then get increased up to len, when equal to len, we stop trying
+}
+
+#[derive(Clone, Debug, PartialEq)]
+pub enum VecMutationStep<S> {
+    InnerMutatorIsUnit { length_step: usize },
+    Normal(MutationStep<S>),
+}
+
+#[derive(Clone, Debug, PartialEq)]
+pub enum VecArbitraryStep {
+    InnerMutatorIsUnit { length_step: usize },
+    Normal { make_empty: bool },
+}
+
+#[derive(Clone, Debug, PartialEq)]
 pub struct MutationStep<S> {
+    executed_mutations: ExecutedMutations,
     inner: Vec<S>,
     alias: Option<VoseAlias>,
 }
@@ -77,6 +96,7 @@ pub struct VecMutatorCache<C> {
     alias: Option<VoseAlias>,
 }
 
+#[doc(hidden)]
 pub enum UnmutateVecToken<T: Clone, M: Mutator<T>> {
     Elements(Vec<(usize, M::UnmutateToken)>),
     Element(usize, M::UnmutateToken),
@@ -94,10 +114,17 @@ pub enum UnmutateVecToken<T: Clone, M: Mutator<T>> {
     Nothing,
 }
 
+#[doc(hidden)]
+#[derive(Clone, Debug, PartialEq)]
+pub struct RecursingPartIndex<RPI> {
+    inner: Vec<RPI>,
+    indices: Vec<usize>,
+}
+
 impl<T: Clone, M: Mutator<T>> VecMutator<T, M> {
     #[no_coverage]
     fn complexity_from_inner(&self, cplx: f64, len: usize) -> f64 {
-        1.0 + cplx + crate::mutators::size_to_cplxity(len.saturating_add(1))
+        1.0 + if cplx <= 0.0 { len as f64 } else { cplx }
     }
     #[no_coverage]
     fn mutate_element(
@@ -134,7 +161,7 @@ impl<T: Clone, M: Mutator<T>> VecMutator<T, M> {
             return None;
         }
         // select disjoint indices here, minimum 2 and maximum 4
-        let nbr_indices = self.rng.usize(2..=cmp::max(2, value.len() - 1));
+        let nbr_indices = self.rng.usize(2..=std::cmp::min(value.len(), 5));
         let mut shuffled_indices: Vec<_> = (0..value.len()).collect();
         self.rng.shuffle(&mut shuffled_indices);
 
@@ -229,6 +256,7 @@ impl<T: Clone, M: Mutator<T>> VecMutator<T, M> {
     #[no_coverage]
     fn remove_element(
         &self,
+
         value: &mut Vec<T>,
         cache: &VecMutatorCache<M::Cache>,
     ) -> Option<(UnmutateVecToken<T, M>, f64)> {
@@ -245,6 +273,22 @@ impl<T: Clone, M: Mutator<T>> VecMutator<T, M> {
         ))
     }
 
+    #[no_coverage]
+    fn unchecked_remove_element(
+        &self,
+        idx: usize,
+        value: &mut Vec<T>,
+        cache: &VecMutatorCache<M::Cache>,
+    ) -> (UnmutateVecToken<T, M>, f64) {
+        let removed_el = value.remove(idx);
+        let removed_el_cplx = self.m.complexity(&removed_el, &cache.inner[idx]);
+        let token = UnmutateVecToken::Insert(idx, removed_el);
+        (
+            token,
+            self.complexity_from_inner(cache.sum_cplx - removed_el_cplx, value.len()),
+        )
+    }
+
     #[no_coverage]
     fn remove_many_elements(
         &self,
@@ -373,21 +417,21 @@ impl<T: Clone, M: Mutator<T>> VecMutator<T, M> {
         #[no_coverage]
         fn length_for_elements_of_cplx(target_cplx: f64, cplx: f64) -> usize {
             if cplx == 0.0 {
+                assert!(target_cplx.is_finite());
+                assert!(target_cplx >= 0.0);
                 // cplx is 0, so the length is the maximum complexity of the length component of the vector
-                crate::mutators::cplxity_to_size(target_cplx - 1.0)
+                target_cplx.round() as usize
             } else if !cplx.is_finite() {
                 0
             } else {
                 (target_cplx / cplx).trunc() as usize
             }
         }
-
         let min_len = length_for_elements_of_cplx(target_cplx, self.m.max_complexity());
         let max_len = length_for_elements_of_cplx(target_cplx, self.m.min_complexity());
 
         let min_len = clamp(&self.len_range, min_len);
         let max_len = clamp(&(min_len..=*self.len_range.end()), max_len);
-
         min_len..=max_len
     }
 
@@ -425,30 +469,38 @@ impl<T: Clone, M: Mutator<T>> VecMutator<T, M> {
     }
 }
 
-impl<T: Clone, M: Mutator<T>> Mutator<Vec<T>> for VecMutator<T, M> {
+impl<T: Clone + 'static, M: Mutator<T>> Mutator<Vec<T>> for VecMutator<T, M> {
+    #[doc(hidden)]
     type Cache = VecMutatorCache<M::Cache>;
-    type MutationStep = MutationStep<M::MutationStep>;
-    type ArbitraryStep = bool; // false: check empty vector, true: random
+    #[doc(hidden)]
+    type MutationStep = VecMutationStep<M::MutationStep>;
+    #[doc(hidden)]
+    type ArbitraryStep = VecArbitraryStep;
+    #[doc(hidden)]
     type UnmutateToken = UnmutateVecToken<T, M>;
 
+    #[doc(hidden)]
     #[no_coverage]
     fn default_arbitrary_step(&self) -> Self::ArbitraryStep {
-        <_>::default()
+        if self.m.max_complexity() == 0.0 {
+            Self::ArbitraryStep::InnerMutatorIsUnit {
+                length_step: *self.len_range.start(),
+            }
+        } else {
+            Self::ArbitraryStep::Normal { make_empty: false }
+        }
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn validate_value(&self, value: &Vec<T>) -> Option<(Self::Cache, Self::MutationStep)> {
         let inner: Vec<_> = value
             .iter()
-            .filter_map(
+            .map(
                 #[no_coverage]
                 |x| self.m.validate_value(x),
             )
-            .collect();
-
-        if inner.len() < value.len() {
-            return None;
-        }
+            .collect::<Option<_>>()?;
 
         let mut inner_caches = Vec::with_capacity(inner.len());
         let mut inner_steps = Vec::with_capacity(inner.len());
@@ -466,10 +518,20 @@ impl<T: Clone, M: Mutator<T>> Mutator<Vec<T>> for VecMutator<T, M> {
             )
             .collect::<Vec<_>>();
 
-        let sum_cplx = cplxs.iter().fold(0.0, |sum_cplx, c| sum_cplx + c);
+        let sum_cplx = cplxs.iter().fold(
+            0.0,
+            #[no_coverage]
+            |sum_cplx, c| sum_cplx + c,
+        );
 
         let alias = if !inner_caches.is_empty() {
-            let mut probabilities = cplxs.into_iter().map(|c| 10. + c).collect::<Vec<_>>();
+            let mut probabilities = cplxs
+                .into_iter()
+                .map(
+                    #[no_coverage]
+                    |c| 10. + c,
+                )
+                .collect::<Vec<_>>();
             let sum_prob = probabilities.iter().sum::<f64>();
             probabilities.iter_mut().for_each(
                 #[no_coverage]
@@ -485,59 +547,88 @@ impl<T: Clone, M: Mutator<T>> Mutator<Vec<T>> for VecMutator<T, M> {
             sum_cplx,
             alias: alias.clone(),
         };
-        let step = MutationStep {
-            inner: inner_steps,
-            alias,
+        let step = if self.m.max_complexity() == 0.0 {
+            Self::MutationStep::InnerMutatorIsUnit {
+                length_step: *self.len_range.start(),
+            }
+        } else {
+            Self::MutationStep::Normal(MutationStep {
+                executed_mutations: ExecutedMutations {
+                    make_empty: value.is_empty(),
+                    remove_element: 0,
+                },
+                inner: inner_steps,
+                alias,
+            })
         };
 
         Some((cache, step))
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn max_complexity(&self) -> f64 {
         let max_len = *self.len_range.end();
         self.complexity_from_inner((max_len as f64) * self.m.max_complexity(), max_len.saturating_add(1))
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn min_complexity(&self) -> f64 {
         let min_len = *self.len_range.start();
-        self.complexity_from_inner((min_len as f64) * self.m.min_complexity(), min_len)
+        if min_len == 0 {
+            return 1.0;
+        } else {
+            self.complexity_from_inner((min_len as f64) * self.m.min_complexity(), min_len)
+        }
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn complexity(&self, value: &Vec<T>, cache: &Self::Cache) -> f64 {
         self.complexity_from_inner(cache.sum_cplx, value.len())
     }
 
+    #[doc(hidden)]
     #[no_coverage]
-    fn ordered_arbitrary(&self, step: &mut Self::ArbitraryStep, mut max_cplx: f64) -> Option<(Vec<T>, f64)> {
+    fn ordered_arbitrary(&self, step: &mut Self::ArbitraryStep, max_cplx: f64) -> Option<(Vec<T>, f64)> {
         if max_cplx < self.min_complexity() {
             return None;
         }
-        let mutator_max_cplx = self.max_complexity();
-        if max_cplx > mutator_max_cplx {
-            max_cplx = mutator_max_cplx;
-        }
-        if !*step || max_cplx <= 1.0 {
-            *step = true;
-            if self.len_range.contains(&0) {
-                Some((<_>::default(), 1.0))
-            } else {
-                Some(self.random_arbitrary(max_cplx))
+        match step {
+            VecArbitraryStep::InnerMutatorIsUnit { length_step } => {
+                if self.len_range.contains(length_step) && (*length_step as f64) < max_cplx {
+                    let mut result = Vec::with_capacity(*length_step);
+                    for _ in 0..*length_step {
+                        let (e, c) = self.m.random_arbitrary(1.0);
+                        assert!(c == 0.0);
+                        result.push(e);
+                    }
+                    let cplx = self.complexity_from_inner(0.0, *length_step);
+                    *length_step += 1;
+                    Some((result, cplx))
+                } else {
+                    None
+                }
+            }
+            VecArbitraryStep::Normal { make_empty } => {
+                if !*make_empty || max_cplx <= 1.0 {
+                    *make_empty = true;
+                    if self.len_range.contains(&0) {
+                        Some((<_>::default(), 1.0))
+                    } else {
+                        Some(self.random_arbitrary(max_cplx))
+                    }
+                } else {
+                    Some(self.random_arbitrary(max_cplx))
+                }
             }
-        } else {
-            Some(self.random_arbitrary(max_cplx))
         }
     }
 
+    #[doc(hidden)]
     #[no_coverage]
-    fn random_arbitrary(&self, mut max_cplx: f64) -> (Vec<T>, f64) {
-        let mutator_max_cplx = self.max_complexity();
-        if max_cplx > mutator_max_cplx {
-            max_cplx = mutator_max_cplx;
-        }
-
+    fn random_arbitrary(&self, max_cplx: f64) -> (Vec<T>, f64) {
         let min_cplx = self.min_complexity();
         if max_cplx <= min_cplx || self.rng.u8(..) == 0 {
             // return the least complex value possible
@@ -551,6 +642,7 @@ impl<T: Clone, M: Mutator<T>> Mutator<Vec<T>> for VecMutator<T, M> {
             let cplx = self.complexity_from_inner(inner_cplx, v.len());
             return (v, cplx);
         }
+
         let target_cplx = crate::mutators::gen_f64(&self.rng, min_cplx..max_cplx);
         let len_range = self.choose_slice_length(target_cplx);
         let upperbound_max_len = std::cmp::min(*len_range.end(), (max_cplx / self.m.min_complexity()).ceil() as usize);
@@ -559,6 +651,7 @@ impl<T: Clone, M: Mutator<T>> Mutator<Vec<T>> for VecMutator<T, M> {
         self.new_input_with_length_and_complexity(target_len, target_cplx)
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn ordered_mutate(
         &self,
@@ -570,73 +663,109 @@ impl<T: Clone, M: Mutator<T>> Mutator<Vec<T>> for VecMutator<T, M> {
         if max_cplx < self.min_complexity() {
             return None;
         }
-        if self.rng.usize(0..100) == 0 {
-            let (mut v, cplx) = self.random_arbitrary(max_cplx);
-            std::mem::swap(value, &mut v);
-            return Some((UnmutateVecToken::Replace(v), cplx));
-        }
-        let mutator_max_cplx = self.max_complexity();
-        if max_cplx > mutator_max_cplx {
-            max_cplx = mutator_max_cplx;
-        }
-        let current_cplx = self.complexity(value, cache);
-        let spare_cplx = max_cplx - current_cplx;
-
-        let token = if value.is_empty() || step.alias.is_none() || self.rng.usize(0..10) == 0 {
-            // vector mutation
-            let end = if value.is_empty() {
-                5
-            } else if self.dictionary.is_empty() {
-                10
-            } else if value.len() < 2 {
-                15
-            } else {
-                18
-            };
-            match self.rng.usize(0..end) {
-                0..=3 => self.insert_element(value, cache, spare_cplx),
-                4 => self.insert_repeated_elements(value, cache, spare_cplx),
-                5..=8 => self.remove_element(value, cache),
-                9 => self.remove_many_elements(value, cache),
-                10..=14 => self.use_dictionary(value, cache, spare_cplx),
-                15 => self.swap_elements(value, cache),
-                16 => self.mutate_many_elements(value, cache, spare_cplx),
-                17 => self.remove_and_insert_element(value, cache, spare_cplx),
-                _ => unreachable!(),
-            }
-        } else {
-            // we know value is not empty, therefore the alias is Some
-            if let Some(alias) = step.alias.as_ref() {
-                let idx = alias.sample();
-                if let Some(x) = self.mutate_element(value, cache, step, idx, spare_cplx) {
-                    Some(x)
-                } else {
-                    let mut prob = step.alias.as_ref().unwrap().original_probabilities.clone();
-                    prob[idx] = 0.0;
-                    let sum = prob.iter().sum::<f64>();
-                    if sum == 0.0 {
-                        step.alias = None;
-                    } else {
-                        prob.iter_mut().for_each(
-                            #[no_coverage]
-                            |c| *c /= sum,
-                        );
-                        step.alias = Some(VoseAlias::new(prob));
+        match step {
+            VecMutationStep::InnerMutatorIsUnit { length_step } => {
+                if self.len_range.contains(length_step) && (*length_step as f64) < max_cplx {
+                    let mut result = Vec::with_capacity(*length_step);
+                    for _ in 0..*length_step {
+                        let (e, c) = self.m.random_arbitrary(1.0);
+                        assert!(c == 0.0);
+                        result.push(e);
                     }
+                    let cplx = self.complexity_from_inner(0.0, *length_step);
+                    *length_step += 1;
+                    std::mem::swap(value, &mut result);
+                    let token = Self::UnmutateToken::Replace(result);
+                    return Some((token, cplx));
+                } else {
+                    return None;
+                }
+            }
+            VecMutationStep::Normal(step) => {
+                if !step.executed_mutations.make_empty && *self.len_range.start() == 0 {
+                    step.executed_mutations.make_empty = true;
+                    let mut old_value = vec![];
+                    std::mem::swap(value, &mut old_value);
+                    return Some((Self::UnmutateToken::Replace(old_value), self.min_complexity()));
+                }
+                if step.executed_mutations.remove_element < value.len()
+                    && !value.is_empty()
+                    && self.len_range.contains(&(value.len() - 1))
+                {
+                    let (t, cplx) = self.unchecked_remove_element(step.executed_mutations.remove_element, value, cache);
+                    step.executed_mutations.remove_element += 1;
+                    return Some((t, cplx));
+                }
 
-                    None
+                if self.rng.usize(0..100) == 0 {
+                    let (mut v, cplx) = self.random_arbitrary(max_cplx);
+                    std::mem::swap(value, &mut v);
+                    return Some((UnmutateVecToken::Replace(v), cplx));
+                }
+                let mutator_max_cplx = self.max_complexity();
+                if max_cplx > mutator_max_cplx {
+                    max_cplx = mutator_max_cplx;
+                }
+                let current_cplx = self.complexity(value, cache);
+                let spare_cplx = max_cplx - current_cplx;
+
+                let token = if value.is_empty() || step.alias.is_none() || self.rng.usize(0..5) == 0 {
+                    // vector mutation
+                    if !self.dictionary.is_empty() && self.rng.usize(..5) == 0 {
+                        self.use_dictionary(value, cache, spare_cplx)
+                    } else {
+                        let end = if value.is_empty() {
+                            8
+                        } else if value.len() < 2 {
+                            8
+                        } else {
+                            20
+                        };
+                        match self.rng.usize(0..end) {
+                            0..=3 => self.insert_element(value, cache, spare_cplx),
+                            4..=7 => self.insert_repeated_elements(value, cache, spare_cplx),
+                            8..=9 => self.remove_many_elements(value, cache),
+                            10 => self.swap_elements(value, cache),
+                            11..=18 => self.mutate_many_elements(value, cache, spare_cplx),
+                            19 => self.remove_and_insert_element(value, cache, spare_cplx),
+                            _ => unreachable!(),
+                        }
+                    }
+                } else {
+                    if let Some(alias) = step.alias.as_ref() {
+                        let idx = alias.sample();
+                        if let Some(x) = self.mutate_element(value, cache, step, idx, spare_cplx) {
+                            Some(x)
+                        } else {
+                            let mut prob = step.alias.as_ref().unwrap().original_probabilities.clone();
+                            prob[idx] = 0.0;
+                            let sum = prob.iter().sum::<f64>();
+                            if sum == 0.0 {
+                                step.alias = None;
+                            } else {
+                                prob.iter_mut().for_each(
+                                    #[no_coverage]
+                                    |c| *c /= sum,
+                                );
+                                step.alias = Some(VoseAlias::new(prob));
+                            }
+
+                            None
+                        }
+                    } else {
+                        None
+                    }
+                };
+                if let Some(token) = token {
+                    Some(token)
+                } else {
+                    Some(self.random_mutate(value, cache, max_cplx))
                 }
-            } else {
-                None
             }
-        };
-        if let Some(token) = token {
-            Some(token)
-        } else {
-            Some(self.random_mutate(value, cache, max_cplx))
         }
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn random_mutate(
         &self,
@@ -648,6 +777,19 @@ impl<T: Clone, M: Mutator<T>> Mutator<Vec<T>> for VecMutator<T, M> {
         if max_cplx > mutator_max_cplx {
             max_cplx = mutator_max_cplx;
         }
+        if self.m.max_complexity() == 0.0 {
+            let length = std::cmp::min(max_cplx.round() as usize, self.rng.usize(self.len_range.clone()));
+            let mut result = Vec::with_capacity(length);
+            for _ in 0..length {
+                let (e, c) = self.m.random_arbitrary(1.0);
+                assert!(c == 0.0);
+                result.push(e);
+            }
+            let cplx = self.complexity_from_inner(0.0, length);
+            std::mem::swap(value, &mut result);
+            let token = Self::UnmutateToken::Replace(result);
+            return (token, cplx);
+        }
         if self.rng.usize(0..100) == 0 {
             let (mut v, cplx) = self.random_arbitrary(max_cplx);
             std::mem::swap(value, &mut v);
@@ -656,27 +798,27 @@ impl<T: Clone, M: Mutator<T>> Mutator<Vec<T>> for VecMutator<T, M> {
         let current_cplx = self.complexity(value, cache);
         let spare_cplx = max_cplx - current_cplx;
 
-        if value.is_empty() || self.rng.usize(0..10) == 0 {
-            // vector mutation
-            let end = if value.is_empty() {
-                5
-            } else if self.dictionary.is_empty() {
-                10
-            } else if value.len() < 2 {
-                15
+        if value.is_empty() || self.rng.usize(0..5) == 0 {
+            if !self.dictionary.is_empty() && self.rng.usize(..5) == 0 {
+                self.use_dictionary(value, cache, spare_cplx)
             } else {
-                18
-            };
-            match self.rng.usize(0..end) {
-                0..=3 => self.insert_element(value, cache, spare_cplx),
-                4 => self.insert_repeated_elements(value, cache, spare_cplx),
-                5..=8 => self.remove_element(value, cache),
-                9 => self.remove_many_elements(value, cache),
-                10..=14 => self.use_dictionary(value, cache, spare_cplx),
-                15 => self.swap_elements(value, cache),
-                16 => self.mutate_many_elements(value, cache, spare_cplx),
-                17 => self.remove_and_insert_element(value, cache, spare_cplx),
-                _ => None,
+                let end = if value.is_empty() {
+                    8
+                } else if value.len() < 2 {
+                    9
+                } else {
+                    20
+                };
+                match self.rng.usize(0..end) {
+                    0..=3 => self.insert_element(value, cache, spare_cplx),
+                    4..=7 => self.insert_repeated_elements(value, cache, spare_cplx),
+                    8 => self.remove_element(value, cache),
+                    9 => self.remove_many_elements(value, cache),
+                    10 => self.swap_elements(value, cache),
+                    11..=18 => self.mutate_many_elements(value, cache, spare_cplx),
+                    19 => self.remove_and_insert_element(value, cache, spare_cplx),
+                    _ => unreachable!(),
+                }
             }
             .unwrap_or_else(
                 #[no_coverage]
@@ -702,6 +844,7 @@ impl<T: Clone, M: Mutator<T>> Mutator<Vec<T>> for VecMutator<T, M> {
         }
     }
 
+    #[doc(hidden)]
     #[no_coverage]
     fn unmutate(&self, value: &mut Vec<T>, cache: &mut Self::Cache, t: Self::UnmutateToken) {
         match t {
@@ -744,6 +887,50 @@ impl<T: Clone, M: Mutator<T>> Mutator<Vec<T>> for VecMutator<T, M> {
             UnmutateVecToken::Nothing => {}
         }
     }
+
+    #[doc(hidden)]
+    type RecursingPartIndex = RecursingPartIndex<M::RecursingPartIndex>;
+    #[doc(hidden)]
+    #[no_coverage]
+    fn default_recursing_part_index(&self, value: &Vec<T>, cache: &Self::Cache) -> Self::RecursingPartIndex {
+        RecursingPartIndex {
+            inner: value
+                .iter()
+                .zip(cache.inner.iter())
+                .map(|(v, c)| self.m.default_recursing_part_index(v, c))
+                .collect(),
+            indices: (0..value.len()).collect(),
+        }
+    }
+    #[doc(hidden)]
+    #[no_coverage]
+    fn recursing_part<'a, V, N>(
+        &self,
+        parent: &N,
+        value: &'a Vec<T>,
+        index: &mut Self::RecursingPartIndex,
+    ) -> Option<&'a V>
+    where
+        V: Clone + 'static,
+        N: Mutator<V>,
+    {
+        assert_eq!(index.inner.len(), index.indices.len());
+        if index.inner.is_empty() {
+            return None;
+        }
+        let choice = self.rng.usize(..index.inner.len());
+        let subindex = &mut index.inner[choice];
+        let value_index = index.indices[choice];
+        let v = &value[value_index];
+        let result = self.m.recursing_part(parent, v, subindex);
+        if result.is_none() {
+            index.inner.remove(choice);
+            index.indices.remove(choice);
+            self.recursing_part::<V, N>(parent, value, index)
+        } else {
+            result
+        }
+    }
 }
 
 #[no_coverage]
@@ -769,7 +956,7 @@ mod tests {
     fn test_constrained_length_mutator_ordered_arbitrary() {
         let range = 0..=10;
         let m = VecMutator::<u8, U8Mutator>::new(U8Mutator::default(), range.clone());
-        let mut step = false;
+        let mut step = m.default_arbitrary_step();
 
         let mut lengths = vec![0; 11];
         let mut cplxs = vec![0; 81];
@@ -789,7 +976,7 @@ mod tests {
     fn test_constrained_length_mutator_ordered_mutate() {
         let range = 0..=10;
         let m = VecMutator::<u8, U8Mutator>::new(U8Mutator::default(), range.clone());
-        let mut step = false;
+        let mut step = m.default_arbitrary_step();
 
         let mut lengths = vec![0; 11];
         let mut cplxs = vec![0; 81];
