@@ -1,52 +1,38 @@
-//! This crate contains types implementing the Serializer trait of fuzzcheck.
-//! There are currently two implementations:
+//! Types implementing the [Serializer] trait.
+//!
+//! There are currently three implementations:
 //!
 //! * SerdeSerializer uses the `serde` and `serde_json` crate to serialize
 //! the test inputs (of arbitrary Serializable type) to a `.json` file.
-//! It is available under the “serde” feature
 //!
 //! * [ByteSerializer] encodes and decodes values of type `Vec<u8>` by simply
 //! copy/pasting the bytes from/to the files. The extension is customizable.
 //!
+//! * [StringSerializer] encodes and decodes values of any type implementing
+//! `FromStr` and `ToString` into utf-8 encoded text files.
 
-#[cfg(feature = "serde_json_serializer")]
 mod serde_serializer;
 use std::{marker::PhantomData, str::FromStr};
 
-#[cfg(feature = "serde_json_serializer")]
 pub use serde_serializer::SerdeSerializer;
 
-#[cfg(feature = "serde_json_alternative_serializer")]
-mod json_serializer;
-#[cfg(feature = "serde_json_alternative_serializer")]
-pub use decent_serde_json_alternative;
-#[cfg(feature = "serde_json_alternative_serializer")]
-pub use json;
-#[cfg(feature = "serde_json_alternative_serializer")]
-pub use json_serializer::JsonSerializer;
-
 use crate::Serializer;
 
 /**
-A Serializer for Vec<u8> that simply copies the bytes from/to the files.
-
-```ignore
-#[no_coverage] fn parse(data: &[u8]) -> bool {
-    // ...
-}
-let mutator = VecMutator<U8Mutator>::default();
-
-// pass the desired extension of the file as argument
-let serializer = ByteSerializer::new("png");
-
-fuzzcheck::launch(parse, mutator, serializer)
-```
+A Serializer for `Vec<u8>` that simply copies the bytes from/to the files.
 */
 pub struct ByteSerializer {
     ext: &'static str,
 }
 
 impl ByteSerializer {
+    /// Create a byte serializer. The only argument is the name of the extension
+    /// that the created files should have. For example:
+    /// ```
+    /// use fuzzcheck::ByteSerializer;
+    ///
+    /// let ser = ByteSerializer::new("png");
+    /// ````
     #[no_coverage]
     pub fn new(ext: &'static str) -> Self {
         Self { ext }
@@ -55,10 +41,7 @@ impl ByteSerializer {
 
 impl crate::traits::Serializer for ByteSerializer {
     type Value = Vec<u8>;
-    #[no_coverage]
-    fn is_utf8(&self) -> bool {
-        false
-    }
+
     #[no_coverage]
     fn extension(&self) -> &str {
         self.ext
@@ -73,6 +56,10 @@ impl crate::traits::Serializer for ByteSerializer {
     }
 }
 
+/**
+A serializer that encodes and decodes values of any type implementing
+`FromStr` and `ToString` into utf-8 encoded text files.
+ */
 pub struct StringSerializer<StringType>
 where
     StringType: ToString + FromStr,
@@ -84,6 +71,14 @@ impl<StringType> StringSerializer<StringType>
 where
     StringType: ToString + FromStr,
 {
+    /// Create a string serializer. The only argument is the name of the extension
+    /// that the created files should have. For example:
+    /// ```
+    /// use fuzzcheck::StringSerializer;
+    ///
+    /// let ser = StringSerializer::<String>::new("txt");
+    /// ````
+    #[no_coverage]
     pub fn new(extension: &'static str) -> Self {
         Self {
             extension,
@@ -96,20 +91,18 @@ where
     StringType: ToString + FromStr,
 {
     type Value = StringType;
-    fn is_utf8(&self) -> bool {
-        true
-    }
 
+    #[no_coverage]
     fn extension(&self) -> &str {
         self.extension
     }
-
+    #[no_coverage]
     fn from_data(&self, data: &[u8]) -> Option<Self::Value> {
         let string = String::from_utf8(data.to_vec()).ok()?;
         let value = Self::Value::from_str(&string).ok()?;
         Some(value)
     }
-
+    #[no_coverage]
     fn to_data(&self, value: &Self::Value) -> Vec<u8> {
         value.to_string().into_bytes()
     }
