@@ -28,7 +28,10 @@ pub unsafe fn set_signal_handlers<F: 'static>(f: F)
 where
     F: Fn(libc::c_int) -> !,
 {
-    SIGNAL_HANDLER = Some(Box::new(move |x| (f)(x)));
+    SIGNAL_HANDLER = Some(Box::new(
+        #[no_coverage]
+        move |x| (f)(x),
+    ));
 
     let stack_size = libc::SIGSTKSZ;
 
