@@ -1,70 +1,127 @@
+use crate::fuzzer::PoolStorageIndex;
+use crate::fuzzer::TerminationStatus;
+use crate::sensors_and_pools::stats::EmptyStats;
 use crate::traits::CorpusDelta;
-use crate::traits::EmptyStats;
-use crate::{fuzzer::TerminationStatus, traits::Serializer};
+use crate::traits::SaveToStatsFolder;
+use crate::CSVField;
+use crate::ToCSV;
 use fuzzcheck_common::arg::Arguments;
 use fuzzcheck_common::arg::FuzzerCommand;
 use fuzzcheck_common::{FuzzerEvent, FuzzerStats};
-use owo_colors::OwoColorize;
+use nu_ansi_term::Color;
+use std::cell::RefCell;
 use std::collections::hash_map::DefaultHasher;
 use std::collections::HashMap;
 use std::fmt::Display;
 use std::fs;
+use std::fs::File;
+use std::fs::OpenOptions;
 use std::hash::{Hash, Hasher};
+use std::io::Write;
 use std::io::{self, Result};
 use std::path::Path;
+use std::path::PathBuf;
 use std::time::Duration;
 use std::time::Instant;
+use std::time::SystemTime;
 
-pub struct World<S: Serializer, CorpusKey: Hash + Eq> {
+impl ToCSV for FuzzerStats {
+    #[no_coverage]
+    fn csv_headers(&self) -> Vec<CSVField> {
+        vec![
+            CSVField::String("nbr_iter".to_string()),
+            CSVField::String("iter/s".to_string()),
+        ]
+    }
+    #[no_coverage]
+    fn to_csv_record(&self) -> Vec<CSVField> {
+        vec![
+            CSVField::Integer(self.total_number_of_runs as isize),
+            CSVField::Integer(self.exec_per_s as isize),
+        ]
+    }
+}
+
+pub struct World {
     settings: Arguments,
     initial_instant: Instant,
     checkpoint_instant: Instant,
-    pub serializer: S,
     /// keeps track of the hash of each input in the corpus, indexed by the Pool key
-    pub corpus: HashMap<CorpusKey, String>,
+    pub corpus: HashMap<(PathBuf, PoolStorageIndex), String>,
+    pub stats: Option<RefCell<File>>,
+    pub stats_folder: Option<PathBuf>,
 }
 
-impl<S: Serializer, CorpusKey: Hash + Eq> World<S, CorpusKey> {
+impl World {
     #[no_coverage]
-    pub fn new(serializer: S, settings: Arguments) -> Self {
-        Self {
+    pub fn new(settings: Arguments) -> Result<Self> {
+        let (stats, stats_folder) = if let Some(stats_folder) = &settings.stats_folder {
+            let now = SystemTime::now();
+            let duration_since_epoch = now.duration_since(SystemTime::UNIX_EPOCH).unwrap();
+            let name = format!("{}", duration_since_epoch.as_millis());
+            let stats_folder = stats_folder.join(name);
+            std::fs::create_dir_all(&stats_folder)?;
+            let path = stats_folder.join("events").with_extension("csv");
+            let file = OpenOptions::new().create_new(true).append(true).open(path)?;
+            (Some(RefCell::new(file)), Some(stats_folder))
+        } else {
+            (None, None)
+        };
+        Ok(Self {
             settings,
             initial_instant: std::time::Instant::now(),
             checkpoint_instant: std::time::Instant::now(),
-            serializer,
             corpus: HashMap::new(),
-        }
+            stats,
+            stats_folder,
+        })
     }
 
     #[no_coverage]
-    fn hash_and_string_of_input(&self, input: &S::Value) -> (String, Vec<u8>) {
-        let input = self.serializer.to_data(input);
+    fn hash(&self, input: &[u8]) -> String {
         let mut hasher = DefaultHasher::new();
         input.hash(&mut hasher);
         let hash = hasher.finish();
         let hash = format!("{:x}", hash);
-        (hash, input)
+        hash
     }
 
     #[no_coverage]
-    pub(crate) fn update_corpus(&mut self, delta: CorpusDelta<S::Value, CorpusKey>) -> Result<()> {
-        let CorpusDelta { path, add, remove } = delta;
-        for to_remove_key in remove {
-            let hash = self.corpus.remove(&to_remove_key).unwrap();
-            self.remove_from_output_corpus(&path, hash.clone())?;
+    pub fn append_stats_file(&self, fields: &[CSVField]) -> Result<()> {
+        if let Some(stats) = &self.stats {
+            let mut stats = stats.try_borrow_mut().unwrap();
+            stats.write(&CSVField::to_bytes(fields))?;
         }
+        Ok(())
+    }
 
-        if let Some((content, key)) = add {
-            let (hash, input) = self.hash_and_string_of_input(&content);
-            let _old = self.corpus.insert(key, hash.clone());
-            self.add_to_output_corpus(&path, hash.clone(), input.clone())?;
+    #[no_coverage]
+    pub(crate) fn update_corpus(
+        &mut self,
+        idx: PoolStorageIndex,
+        content: Vec<u8>,
+        deltas: &[CorpusDelta],
+        extension: &str,
+    ) -> Result<()> {
+        for delta in deltas {
+            let CorpusDelta { path, add, remove } = delta;
+            for to_remove_key in remove {
+                let hash = self.corpus.remove(&(path.to_path_buf(), *to_remove_key)).unwrap();
+                self.remove_from_output_corpus(&path, hash.clone(), extension)?;
+            }
+
+            if *add {
+                let hash = self.hash(&content);
+                let _old = self.corpus.insert((path.to_path_buf(), idx), hash.clone());
+                self.add_to_output_corpus(&path, hash.clone(), content.clone(), extension)?;
+            }
         }
 
         Ok(())
     }
 
     #[no_coverage]
-    pub fn add_to_output_corpus(&self, path: &Path, name: String, content: Vec<u8>) -> Result<()> {
+    pub fn add_to_output_corpus(&self, path: &Path, name: String, content: Vec<u8>, extension: &str) -> Result<()> {
         if self.settings.corpus_out.is_none() {
             return Ok(());
         }
@@ -74,39 +131,50 @@ impl<S: Serializer, CorpusKey: Hash + Eq> World<S, CorpusKey> {
             std::fs::create_dir_all(&folder)?;
         }
 
-        let path = folder.join(name).with_extension(self.serializer.extension());
+        let path = folder.join(name).with_extension(extension);
         fs::write(path, content)?;
 
         Ok(())
     }
 
     #[no_coverage]
-    pub fn remove_from_output_corpus(&self, path: &Path, name: String) -> Result<()> {
+    pub fn remove_from_output_corpus(&self, path: &Path, name: String, extension: &str) -> Result<()> {
         if self.settings.corpus_out.is_none() {
             return Ok(());
         }
         let corpus = self.settings.corpus_out.as_ref().unwrap().as_path().join(path);
 
-        let path = corpus.join(name).with_extension(self.serializer.extension());
+        let path = corpus.join(name).with_extension(extension);
         let _ = fs::remove_file(path);
 
         Ok(())
     }
 
     #[no_coverage]
-    pub(crate) fn report_event<PoolStats: Display>(
+    pub(crate) fn report_event<PoolStats: Display + ToCSV>(
         &self,
         event: FuzzerEvent,
-        stats: Option<(&FuzzerStats, PoolStats)>,
+        stats: Option<(&FuzzerStats, &PoolStats)>,
     ) {
         // println uses a lock, which may mess up the signal handling
+        let time_since_start = self.initial_instant.elapsed();
+        let time_since_start_display = {
+            let time_since_start_millis = time_since_start.as_millis();
+            if time_since_start_millis > 10_000 {
+                let time_since_start_seconds = time_since_start.as_secs();
+                format!("{}s ", time_since_start_seconds)
+            } else {
+                format!("{}ms ", time_since_start_millis)
+            }
+        };
+        print!("{} ", time_since_start_display);
         match event {
             FuzzerEvent::Start => {
-                println!("{}", "START".yellow());
+                println!("{}", Color::Yellow.paint("START"));
                 return;
             }
             FuzzerEvent::Pulse => {
-                print!("{}\t", "PULSE".yellow());
+                print!("{} ", Color::Yellow.paint("PULSE"));
             }
             FuzzerEvent::Stop => {
                 println!("\n======================== STOPPED ========================");
@@ -114,7 +182,6 @@ impl<S: Serializer, CorpusKey: Hash + Eq> World<S, CorpusKey> {
                 return;
             }
             FuzzerEvent::End => {
-                //;
                 println!("\n======================== END ========================");
                 println!(
                     r#"Fuzzcheck cannot generate more arbitrary values of the input type. This may be
@@ -124,7 +191,6 @@ because the mutator does not know how to generate more values."#
                 return;
             }
             FuzzerEvent::CrashNoInput => {
-                //;
                 println!("\n=================== CRASH DETECTED ===================");
                 println!(
                     r#"A crash was detected, but the fuzzer cannot recover the crashing input.
@@ -133,13 +199,11 @@ This should never happen, and is probably a bug in fuzzcheck. Sorry :("#
                 return;
             }
             FuzzerEvent::Done => {
-                println!("{}", "DONE".yellow());
+                println!("{}", Color::Yellow.paint("DONE"));
                 return;
             }
-            FuzzerEvent::New => print!("{}\t", "NEW".yellow()),
-            FuzzerEvent::Remove(count) => print!("{} {}\t", "REMOVE".yellow(), count.yellow()),
             FuzzerEvent::DidReadCorpus => {
-                println!("{}", "FINISHED READING CORPUS".yellow());
+                println!("{}", Color::Yellow.paint("FINISHED READING CORPUS"));
                 return;
             }
             FuzzerEvent::CaughtSignal(signal) => println!("\n================ SIGNAL {} ================", signal),
@@ -147,24 +211,33 @@ This should never happen, and is probably a bug in fuzzcheck. Sorry :("#
             FuzzerEvent::TestFailure => {
                 println!("\n================ TEST FAILED ================");
             }
-            FuzzerEvent::Replace(count) => {
-                print!("{} {}\t", "RPLC".yellow(), count.yellow());
-            }
+            FuzzerEvent::Replace(_, _) => {}
             FuzzerEvent::None => return,
         };
         if let Some((fuzzer_stats, pool_stats)) = stats {
-            print!("{} ", fuzzer_stats.total_number_of_runs.yellow());
-            print!("{} ", pool_stats.yellow());
-            print!("{} {}", "iter/s:".yellow(), fuzzer_stats.exec_per_s.yellow());
+            print!(
+                "{} ",
+                Color::Yellow.paint(format!("{}", fuzzer_stats.total_number_of_runs))
+            );
+            print!("{} ", Color::Yellow.paint(format!("{}", pool_stats)));
+            print!(
+                "{} ",
+                Color::Yellow.paint(format!("iter/s {}", fuzzer_stats.exec_per_s))
+            );
 
             println!();
+            let mut stats_fields = vec![CSVField::Integer(time_since_start.as_millis() as isize)];
+            stats_fields.extend(fuzzer_stats.to_csv_record());
+            stats_fields.extend(pool_stats.to_csv_record());
+            self.append_stats_file(&stats_fields)
+                .expect("cannot write to stats file");
         }
     }
 
-    #[no_coverage]
-    pub fn set_start_instant(&mut self) {
-        self.initial_instant = Instant::now();
-    }
+    // #[no_coverage]
+    // pub fn set_start_instant(&mut self) {
+    //     self.initial_instant = Instant::now();
+    // }
     #[no_coverage]
     pub fn set_checkpoint_instant(&mut self) {
         self.checkpoint_instant = Instant::now();
@@ -179,7 +252,7 @@ This should never happen, and is probably a bug in fuzzcheck. Sorry :("#
     }
 
     #[no_coverage]
-    pub fn read_input_corpus(&self) -> Result<Vec<S::Value>> {
+    pub fn read_input_corpus(&self) -> Result<Vec<Vec<u8>>> {
         if self.settings.corpus_in.is_none() {
             return Result::Ok(vec![]);
         }
@@ -188,7 +261,8 @@ This should never happen, and is probably a bug in fuzzcheck. Sorry :("#
         self.read_input_corpus_rec(corpus, &mut values)?;
         Ok(values)
     }
-    fn read_input_corpus_rec(&self, corpus: &Path, values: &mut Vec<S::Value>) -> Result<()> {
+    #[no_coverage]
+    fn read_input_corpus_rec(&self, corpus: &Path, values: &mut Vec<Vec<u8>>) -> Result<()> {
         if !corpus.exists() {
             return Ok(());
         }
@@ -205,29 +279,20 @@ This should never happen, and is probably a bug in fuzzcheck. Sorry :("#
                 self.read_input_corpus_rec(&path, values)?;
             } else {
                 let data = fs::read(path)?;
-                if let Some(i) = self.serializer.from_data(&data) {
-                    values.push(i);
-                }
+                values.push(data);
             }
         }
         Ok(())
     }
 
     #[no_coverage]
-    pub fn read_input_file(&self, file: &Path) -> Result<S::Value> {
+    pub fn read_input_file(&self, file: &Path) -> Result<Vec<u8>> {
         let data = fs::read(file)?;
-        if let Some(input) = self.serializer.from_data(&data) {
-            Ok(input)
-        } else {
-            Result::Err(io::Error::new(
-                io::ErrorKind::Other,
-                "The file could not be decoded into a valid input.",
-            ))
-        }
+        Ok(data)
     }
 
     #[no_coverage]
-    pub fn save_artifact(&mut self, input: &S::Value, cplx: f64) -> Result<()> {
+    pub fn save_artifact(&mut self, content: Vec<u8>, cplx: f64, extension: &str) -> Result<()> {
         let artifacts_folder = self.settings.artifacts_folder.as_ref();
         if artifacts_folder.is_none() {
             return Ok(());
@@ -239,7 +304,6 @@ This should never happen, and is probably a bug in fuzzcheck. Sorry :("#
         }
 
         let mut hasher = DefaultHasher::new();
-        let content = self.serializer.to_data(input);
         content.hash(&mut hasher);
         let hash = hasher.finish();
 
@@ -249,7 +313,7 @@ This should never happen, and is probably a bug in fuzzcheck. Sorry :("#
             format!("{:x}", hash)
         };
 
-        let path = artifacts_folder.join(&name).with_extension(self.serializer.extension());
+        let path = artifacts_folder.join(&name).with_extension(extension);
         println!("Failing test case found. Saving at {:?}", path);
         fs::write(path, &content)?;
 
@@ -261,4 +325,22 @@ This should never happen, and is probably a bug in fuzzcheck. Sorry :("#
         self.report_event::<EmptyStats>(FuzzerEvent::Stop, None);
         std::process::exit(TerminationStatus::Success as i32);
     }
+
+    #[no_coverage]
+    pub fn write_stats_content(&self, contents: Vec<(PathBuf, Vec<u8>)>) -> Result<()> {
+        if let Some(stats_folder) = &self.stats_folder {
+            for (path, content) in contents {
+                let path = stats_folder.join(path);
+                fs::write(path, &content)?;
+            }
+        }
+        Ok(())
+    }
+}
+impl SaveToStatsFolder for World {
+    #[no_coverage]
+    fn save_to_stats_folder(&self) -> Vec<(PathBuf, Vec<u8>)> {
+        let content = serde_json::to_vec(&self.corpus.iter().collect::<Vec<_>>()).unwrap();
+        vec![(PathBuf::new().join("world.json"), content)]
+    }
 }
